from .common import TCP_PLATFORM_SCHEMA as TCP_PLATFORM_SCHEMA, TcpEntity as TcpEntity
from .const import CONF_VALUE_ON as CONF_VALUE_ON
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Final

PLATFORM_SCHEMA: Final[Any]

def setup_platform(hass: HomeAssistant, config: ConfigType, add_entities: AddEntitiesCallback, discovery_info: Union[dict[str, Any], None] = ...) -> None: ...

class TcpBinarySensor(TcpEntity, BinarySensorEntity):
    @property
    def is_on(self) -> bool: ...



from homeassistant.const import CONF_UNIT_SYSTEM_IMPERIAL as CONF_UNIT_SYSTEM_IMPERIAL, CONF_UNIT_SYSTEM_METRIC as CONF_UNIT_SYSTEM_METRIC, LENGTH as LENGTH, LENGTH_KILOMETERS as LENGTH_KILOMETERS, LENGTH_MILES as LENGTH_MILES, MASS as MASS, MASS_GRAMS as MASS_GRAMS, MASS_KILOGRAMS as MASS_KILOGRAMS, MASS_OUNCES as MASS_OUNCES, MASS_POUNDS as MASS_POUNDS, PRESSURE as PRESSURE, PRESSURE_PA as PRESSURE_PA, PRESSURE_PSI as PRESSURE_PSI, TEMPERATURE as TEMPERATURE, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT, UNIT_NOT_RECOGNIZED_TEMPLATE as UNIT_NOT_RECOGNIZED_TEMPLATE, VOLUME as VOLUME, VOLUME_GALLONS as VOLUME_GALLONS, VOLUME_LITERS as VOLUME_LITERS
from typing import Any

LENGTH_UNITS: Any
MASS_UNITS: tuple[str, ...]
PRESSURE_UNITS: Any
VOLUME_UNITS: Any
TEMPERATURE_UNITS: tuple[str, ...]

def is_valid_unit(unit: str, unit_type: str) -> bool: ...

class UnitSystem:
    name: Any
    temperature_unit: Any
    length_unit: Any
    mass_unit: Any
    pressure_unit: Any
    volume_unit: Any
    def __init__(self, name: str, temperature: str, length: str, volume: str, mass: str, pressure: str) -> None: ...
    @property
    def is_metric(self) -> bool: ...
    def temperature(self, temperature: float, from_unit: str) -> float: ...
    def length(self, length: Union[float, None], from_unit: str) -> float: ...
    def pressure(self, pressure: Union[float, None], from_unit: str) -> float: ...
    def volume(self, volume: Union[float, None], from_unit: str) -> float: ...
    def as_dict(self) -> dict[str, str]: ...

METRIC_SYSTEM: Any
IMPERIAL_SYSTEM: Any



from . import RitualsDataUpdateCoordinator as RitualsDataUpdateCoordinator
from .const import COORDINATORS as COORDINATORS, DEVICES as DEVICES, DOMAIN as DOMAIN
from .entity import DiffuserEntity as DiffuserEntity
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity, DEVICE_CLASS_BATTERY_CHARGING as DEVICE_CLASS_BATTERY_CHARGING
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from pyrituals import Diffuser as Diffuser
from typing import Any

CHARGING_SUFFIX: str

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class DiffuserBatteryChargingBinarySensor(DiffuserEntity, BinarySensorEntity):
    _attr_device_class: Any
    def __init__(self, diffuser: Diffuser, coordinator: RitualsDataUpdateCoordinator) -> None: ...
    @property
    def is_on(self) -> bool: ...



from .const import API_URL as API_URL, OAUTH2_TOKEN as OAUTH2_TOKEN, SDM_SCOPES as SDM_SCOPES
from aiohttp import ClientSession as ClientSession
from google.oauth2.credentials import Credentials
from google_nest_sdm.auth import AbstractAuth
from homeassistant.helpers import config_entry_oauth2_flow as config_entry_oauth2_flow
from typing import Any

class AsyncConfigEntryAuth(AbstractAuth):
    _oauth_session: Any
    _client_id: Any
    _client_secret: Any
    def __init__(self, websession: ClientSession, oauth_session: config_entry_oauth2_flow.OAuth2Session, client_id: str, client_secret: str) -> None: ...
    async def async_get_access_token(self) -> str: ...
    async def async_get_creds(self) -> Credentials: ...



import upcloud_api
from .const import CONFIG_ENTRY_UPDATE_SIGNAL_TEMPLATE as CONFIG_ENTRY_UPDATE_SIGNAL_TEMPLATE, DEFAULT_SCAN_INTERVAL as DEFAULT_SCAN_INTERVAL, DOMAIN as DOMAIN
from datetime import timedelta
from homeassistant.config_entries import ConfigEntry as ConfigEntry, SOURCE_IMPORT as SOURCE_IMPORT
from homeassistant.const import CONF_PASSWORD as CONF_PASSWORD, CONF_SCAN_INTERVAL as CONF_SCAN_INTERVAL, CONF_USERNAME as CONF_USERNAME, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON, STATE_PROBLEM as STATE_PROBLEM
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect, async_dispatcher_send as async_dispatcher_send
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity, DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any, Dict

_LOGGER: Any
ATTR_CORE_NUMBER: str
ATTR_HOSTNAME: str
ATTR_MEMORY_AMOUNT: str
ATTR_TITLE: str
ATTR_UUID: str
ATTR_ZONE: str
CONF_SERVERS: str
DATA_UPCLOUD: str
DEFAULT_COMPONENT_NAME: str
DEFAULT_COMPONENT_DEVICE_CLASS: str
CONFIG_ENTRY_DOMAINS: Any
SIGNAL_UPDATE_UPCLOUD: str
STATE_MAP: Any
CONFIG_SCHEMA: Any

class UpCloudDataUpdateCoordinator(DataUpdateCoordinator[Dict[str, upcloud_api.Server]]):
    cloud_manager: Any
    def __init__(self, hass: HomeAssistant, cloud_manager: upcloud_api.CloudManager, update_interval: timedelta, username: str) -> None: ...
    update_interval: Any
    async def async_update_config(self, config_entry: ConfigEntry) -> None: ...
    async def _async_update_data(self) -> dict[str, upcloud_api.Server]: ...

class UpCloudHassData:
    coordinators: dict[str, UpCloudDataUpdateCoordinator]
    scan_interval_migrations: dict[str, int]

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
def _config_entry_update_signal_name(config_entry: ConfigEntry) -> str: ...
async def _async_signal_options_update(hass: HomeAssistant, config_entry: ConfigEntry) -> None: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool: ...

class UpCloudServerEntity(CoordinatorEntity):
    _attr_device_class: Any
    uuid: Any
    def __init__(self, coordinator: DataUpdateCoordinator[dict[str, upcloud_api.Server]], uuid: str) -> None: ...
    @property
    def _server(self) -> upcloud_api.Server: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def icon(self) -> str: ...
    @property
    def is_on(self) -> bool: ...
    @property
    def available(self) -> bool: ...
    @property
    def extra_state_attributes(self) -> dict[str, Any]: ...



from .const import ATTR_MAX as ATTR_MAX, ATTR_MIN as ATTR_MIN, ATTR_STEP as ATTR_STEP, ATTR_VALUE as ATTR_VALUE, DEFAULT_MAX_VALUE as DEFAULT_MAX_VALUE, DEFAULT_MIN_VALUE as DEFAULT_MIN_VALUE, DEFAULT_STEP as DEFAULT_STEP, DOMAIN as DOMAIN, SERVICE_SET_VALUE as SERVICE_SET_VALUE
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE
from homeassistant.helpers.entity import Entity as Entity, EntityDescription as EntityDescription
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

SCAN_INTERVAL: Any
ENTITY_ID_FORMAT: Any
MIN_TIME_BETWEEN_SCANS: Any
_LOGGER: Any

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class NumberEntityDescription(EntityDescription): ...

class NumberEntity(Entity):
    entity_description: NumberEntityDescription
    _attr_max_value: float
    _attr_min_value: float
    _attr_state: None
    _attr_step: float
    _attr_value: float
    @property
    def capability_attributes(self) -> dict[str, Any]: ...
    @property
    def min_value(self) -> float: ...
    @property
    def max_value(self) -> float: ...
    @property
    def step(self) -> float: ...
    @property
    def state(self) -> Union[float, None]: ...
    @property
    def value(self) -> Union[float, None]: ...
    def set_value(self, value: float) -> None: ...
    async def async_set_value(self, value: float) -> None: ...



from . import EsphomeEntity as EsphomeEntity, platform_async_setup_entry as platform_async_setup_entry
from aioesphomeapi import CameraInfo, CameraState
from aiohttp import web as web
from homeassistant.components import camera as camera
from homeassistant.components.camera import Camera as Camera
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class EsphomeCamera(Camera, EsphomeEntity[CameraInfo, CameraState]):
    _image_cond: Any
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def _on_state_update(self) -> None: ...
    async def _on_state_update_coro(self) -> None: ...
    async def async_camera_image(self) -> Union[bytes, None]: ...
    async def _async_camera_stream_image(self) -> Union[bytes, None]: ...
    async def handle_async_mjpeg_stream(self, request: web.Request) -> web.StreamResponse: ...



import logging
import voluptuous as vol
from .entity_platform import EntityPlatform as EntityPlatform
from collections.abc import Iterable
from datetime import timedelta
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_ENTITY_NAMESPACE as CONF_ENTITY_NAMESPACE, CONF_SCAN_INTERVAL as CONF_SCAN_INTERVAL, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, ServiceCall as ServiceCall, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers import config_per_platform as config_per_platform, discovery as discovery, entity as entity, service as service
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from homeassistant.loader import async_get_integration as async_get_integration, bind_hass as bind_hass
from homeassistant.setup import async_prepare_setup_platform as async_prepare_setup_platform
from types import ModuleType
from typing import Any, Callable

DEFAULT_SCAN_INTERVAL: Any
DATA_INSTANCES: str

async def async_update_entity(hass: HomeAssistant, entity_id: str) -> None: ...

class EntityComponent:
    logger: Any
    hass: Any
    domain: Any
    scan_interval: Any
    config: Any
    _platforms: Any
    async_add_entities: Any
    add_entities: Any
    def __init__(self, logger: logging.Logger, domain: str, hass: HomeAssistant, scan_interval: timedelta = ...) -> None: ...
    @property
    def entities(self) -> Iterable[entity.Entity]: ...
    def get_entity(self, entity_id: str) -> Union[entity.Entity, None]: ...
    def setup(self, config: ConfigType) -> None: ...
    async def async_setup(self, config: ConfigType) -> None: ...
    async def async_setup_entry(self, config_entry: ConfigEntry) -> bool: ...
    async def async_unload_entry(self, config_entry: ConfigEntry) -> bool: ...
    async def async_extract_from_service(self, service_call: ServiceCall, expand_group: bool = ...) -> list[entity.Entity]: ...
    def async_register_entity_service(self, name: str, schema: Union[dict[str, Any], vol.Schema], func: Union[str, Callable[..., Any]], required_features: Union[list[int], None] = ...) -> None: ...
    async def async_setup_platform(self, platform_type: str, platform_config: ConfigType, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...
    async def _async_reset(self) -> None: ...
    async def async_remove_entity(self, entity_id: str) -> None: ...
    async def async_prepare_reload(self, *, skip_reset: bool = ...) -> Union[ConfigType, None]: ...
    def _async_init_entity_platform(self, platform_type: str, platform: Union[ModuleType, None], scan_interval: Union[timedelta, None] = ..., entity_namespace: Union[str, None] = ...) -> EntityPlatform: ...
    async def _async_shutdown(self, event: Event) -> None: ...



from canary.api import Location as Location
from collections.abc import ValuesView
from typing import List, Optional, Tuple, TypedDict

class CanaryData(TypedDict):
    locations: dict[str, Location]
    readings: dict[str, ValuesView]
SensorTypeItem = Tuple[str, Optional[str], Optional[str], Optional[str], List[str]]



from .const import CONF_UID as CONF_UID, DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant import config_entries as config_entries
from homeassistant.components.dhcp import IP_ADDRESS as IP_ADDRESS
from homeassistant.const import CONF_IP_ADDRESS as CONF_IP_ADDRESS, CONF_PORT as CONF_PORT
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.typing import DiscoveryInfoType as DiscoveryInfoType
from typing import Any

DEFAULT_PORT: int
DATA_SCHEMA: Any
UNIQUE_ID: str

def async_get_pin_from_discovery_hostname(hostname: str) -> str: ...
def async_get_pin_from_uid(uid: str) -> str: ...
async def validate_input(hass: HomeAssistant, data: dict[str, Any]) -> dict[str, Any]: ...

class ConfigFlow(config_entries.ConfigFlow):
    VERSION: int
    discovery_info: Any
    def __init__(self) -> None: ...
    async def _async_set_unique_id(self, pin: str) -> None: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_dhcp(self, discovery_info: DiscoveryInfoType) -> FlowResult: ...
    async def async_step_zeroconf(self, discovery_info: DiscoveryInfoType) -> FlowResult: ...
    async def _async_handle_discovery(self) -> FlowResult: ...
    async def async_step_discovery_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



from homeassistant.const import CONF_ID as CONF_ID, CONF_PLATFORM as CONF_PLATFORM
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.loader import IntegrationNotFound as IntegrationNotFound, async_get_integration as async_get_integration
from types import MappingProxyType
from typing import Any, Callable

_PLATFORM_ALIASES: Any

async def _async_get_trigger_platform(hass: HomeAssistant, config: ConfigType) -> Any: ...
async def async_validate_trigger_config(hass: HomeAssistant, trigger_config: list[ConfigType]) -> list[ConfigType]: ...
async def async_initialize_triggers(hass: HomeAssistant, trigger_config: list[ConfigType], action: Callable, domain: str, name: str, log_cb: Callable, home_assistant_start: bool = ..., variables: Union[dict[str, Any], MappingProxyType, None] = ...) -> Union[CALLBACK_TYPE, None]: ...



import ifaddr
from .const import MDNS_TARGET_IP as MDNS_TARGET_IP
from .models import Adapter as Adapter, IPv4ConfiguredAddress as IPv4ConfiguredAddress, IPv6ConfiguredAddress as IPv6ConfiguredAddress
from homeassistant.core import callback as callback
from ipaddress import IPv4Address, IPv6Address
from typing import Any

_LOGGER: Any

async def async_load_adapters() -> list[Adapter]: ...
def enable_adapters(adapters: list[Adapter], enabled_interfaces: list[str]) -> bool: ...
def enable_auto_detected_adapters(adapters: list[Adapter]) -> None: ...
def adapters_with_exernal_addresses(adapters: list[Adapter]) -> list[str]: ...
def _adapter_has_external_address(adapter: Adapter) -> bool: ...
def _has_external_address(ip_str: str) -> bool: ...
def _ip_address_is_external(ip_addr: Union[IPv4Address, IPv6Address]) -> bool: ...
def _reset_enabled_adapters(adapters: list[Adapter]) -> None: ...
def _ifaddr_adapter_to_ha(adapter: ifaddr.Adapter, next_hop_address: Union[None, IPv4Address, IPv6Address]) -> Adapter: ...
def _ip_v6_from_adapter(ip_config: ifaddr.IP) -> IPv6ConfiguredAddress: ...
def _ip_v4_from_adapter(ip_config: ifaddr.IP) -> IPv4ConfiguredAddress: ...
def async_get_source_ip(target_ip: str) -> Union[str, None]: ...



from .const import ATTR_EVENT_TYPE as ATTR_EVENT_TYPE, ATTR_FACE_URL as ATTR_FACE_URL, ATTR_IS_KNOWN as ATTR_IS_KNOWN, ATTR_PERSONS as ATTR_PERSONS, DATA_DEVICE_IDS as DATA_DEVICE_IDS, DATA_PERSONS as DATA_PERSONS, DEFAULT_PERSON as DEFAULT_PERSON, DOMAIN as DOMAIN, EVENT_ID_MAP as EVENT_ID_MAP, NETATMO_EVENT as NETATMO_EVENT
from aiohttp.web import Request as Request
from homeassistant.const import ATTR_DEVICE_ID as ATTR_DEVICE_ID, ATTR_ID as ATTR_ID, ATTR_NAME as ATTR_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.dispatcher import async_dispatcher_send as async_dispatcher_send
from typing import Any

_LOGGER: Any
SUBEVENT_TYPE_MAP: Any

async def async_handle_webhook(hass: HomeAssistant, webhook_id: str, request: Request) -> None: ...
def async_evaluate_event(hass: HomeAssistant, event_data: dict) -> None: ...
def async_send_event(hass: HomeAssistant, event_type: str, data: dict) -> None: ...



from aiohttp import web
from collections.abc import Iterable
from datetime import datetime as dt
from homeassistant.components import websocket_api as websocket_api
from homeassistant.components.http import HomeAssistantView as HomeAssistantView
from homeassistant.components.recorder import history as history, models as history_models
from homeassistant.components.recorder.statistics import list_statistic_ids as list_statistic_ids, statistics_during_period as statistics_during_period
from homeassistant.components.recorder.util import session_scope as session_scope
from homeassistant.const import CONF_DOMAINS as CONF_DOMAINS, CONF_ENTITIES as CONF_ENTITIES, CONF_EXCLUDE as CONF_EXCLUDE, CONF_INCLUDE as CONF_INCLUDE, HTTP_BAD_REQUEST as HTTP_BAD_REQUEST
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.deprecation import deprecated_class as deprecated_class, deprecated_function as deprecated_function
from homeassistant.helpers.entityfilter import CONF_ENTITY_GLOBS as CONF_ENTITY_GLOBS, INCLUDE_EXCLUDE_BASE_FILTER_SCHEMA as INCLUDE_EXCLUDE_BASE_FILTER_SCHEMA
from typing import Any

_LOGGER: Any
DOMAIN: str
CONF_ORDER: str
GLOB_TO_SQL_CHARS: Any
CONFIG_SCHEMA: Any

def get_significant_states(hass, *args, **kwargs): ...
def state_changes_during_period(hass, start_time, end_time: Any | None = ..., entity_id: Any | None = ...): ...
def get_last_state_changes(hass, number_of_states, entity_id): ...
def get_states(hass, utc_point_in_time, entity_ids: Any | None = ..., run: Any | None = ..., filters: Any | None = ...): ...
def get_state(hass, utc_point_in_time, entity_id, run: Any | None = ...): ...
async def async_setup(hass, config): ...

class LazyState(history_models.LazyState): ...

async def ws_get_statistics_during_period(hass: HomeAssistant, connection: websocket_api.ActiveConnection, msg: dict) -> None: ...
async def ws_get_list_statistic_ids(hass: HomeAssistant, connection: websocket_api.ActiveConnection, msg: dict) -> None: ...

class HistoryPeriodView(HomeAssistantView):
    url: str
    name: str
    extra_urls: Any
    filters: Any
    use_include_order: Any
    def __init__(self, filters, use_include_order) -> None: ...
    async def get(self, request: web.Request, datetime: Union[str, None] = ...) -> web.Response: ...
    def _sorted_significant_states_json(self, hass, start_time, end_time, entity_ids, include_start_time_state, significant_changes_only, minimal_response): ...

def sqlalchemy_filter_from_include_exclude_conf(conf): ...

class Filters:
    excluded_entities: Any
    excluded_domains: Any
    excluded_entity_globs: Any
    included_entities: Any
    included_domains: Any
    included_entity_globs: Any
    def __init__(self) -> None: ...
    def apply(self, query): ...
    @property
    def has_config(self): ...
    def bake(self, baked_query): ...
    def entity_filter(self): ...

def _glob_to_like(glob_str): ...
def _entities_may_have_state_changes_after(hass: HomeAssistant, entity_ids: Iterable, start_time: dt) -> bool: ...



from ...config_entries import ConfigEntry as ConfigEntry
from ...helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from .const import DiscoveryInfo as DiscoveryInfo, MYSENSORS_DISCOVERY as MYSENSORS_DISCOVERY, SERVICE_SEND_IR_CODE as SERVICE_SEND_IR_CODE, SensorType as SensorType
from .device import MySensorsDevice as MySensorsDevice
from .helpers import on_unload as on_unload
from homeassistant.components import mysensors as mysensors
from homeassistant.components.switch import DOMAIN as DOMAIN, SwitchEntity as SwitchEntity
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON
from homeassistant.core import HomeAssistant as HomeAssistant, ServiceCall as ServiceCall
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

ATTR_IR_CODE: str
SEND_IR_CODE_SERVICE_SCHEMA: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class MySensorsSwitch(mysensors.device.MySensorsEntity, SwitchEntity):
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...

class MySensorsIRSwitch(MySensorsSwitch):
    _ir_code: Any
    def __init__(self, *args: Any) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def async_update(self) -> None: ...



from homeassistant.const import DEVICE_CLASS_CURRENT as DEVICE_CLASS_CURRENT, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_ILLUMINANCE as DEVICE_CLASS_ILLUMINANCE, DEVICE_CLASS_PRESSURE as DEVICE_CLASS_PRESSURE, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, DEVICE_CLASS_VOLTAGE as DEVICE_CLASS_VOLTAGE, ELECTRIC_CURRENT_AMPERE as ELECTRIC_CURRENT_AMPERE, ELECTRIC_POTENTIAL_VOLT as ELECTRIC_POTENTIAL_VOLT, LIGHT_LUX as LIGHT_LUX, PERCENTAGE as PERCENTAGE, PRESSURE_MBAR as PRESSURE_MBAR, TEMP_CELSIUS as TEMP_CELSIUS
from typing import Any

CONF_MOUNT_DIR: str
CONF_NAMES: str
CONF_TYPE_OWSERVER: str
CONF_TYPE_SYSBUS: str
DEFAULT_OWSERVER_HOST: str
DEFAULT_OWSERVER_PORT: int
DEFAULT_SYSBUS_MOUNT_DIR: str
DOMAIN: str
PRESSURE_CBAR: str
SENSOR_TYPE_COUNT: str
SENSOR_TYPE_CURRENT: str
SENSOR_TYPE_HUMIDITY: str
SENSOR_TYPE_ILLUMINANCE: str
SENSOR_TYPE_MOISTURE: str
SENSOR_TYPE_PRESSURE: str
SENSOR_TYPE_SENSED: str
SENSOR_TYPE_TEMPERATURE: str
SENSOR_TYPE_VOLTAGE: str
SENSOR_TYPE_WETNESS: str
SWITCH_TYPE_LATCH: str
SWITCH_TYPE_PIO: str
SENSOR_TYPES: dict[str, list[Union[str, None]]]
PLATFORMS: Any



from aiohttp import BasicAuth
from homeassistant.components.notify import ATTR_DATA as ATTR_DATA, ATTR_TARGET as ATTR_TARGET, ATTR_TITLE as ATTR_TITLE, BaseNotificationService as BaseNotificationService, PLATFORM_SCHEMA as PLATFORM_SCHEMA
from homeassistant.const import ATTR_ICON as ATTR_ICON, CONF_API_KEY as CONF_API_KEY, CONF_ICON as CONF_ICON, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import aiohttp_client as aiohttp_client, template as template
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from slack import WebClient
from typing import Any, TypedDict

_LOGGER: Any
ATTR_BLOCKS: str
ATTR_BLOCKS_TEMPLATE: str
ATTR_FILE: str
ATTR_PASSWORD: str
ATTR_PATH: str
ATTR_URL: str
ATTR_USERNAME: str
CONF_DEFAULT_CHANNEL: str
DEFAULT_TIMEOUT_SECONDS: int
FILE_PATH_SCHEMA: Any
FILE_URL_SCHEMA: Any
DATA_FILE_SCHEMA: Any
DATA_TEXT_ONLY_SCHEMA: Any
DATA_SCHEMA: Any

class AuthDictT(TypedDict):
    auth: BasicAuth

class FormDataT(TypedDict):
    channels: str
    filename: str
    initial_comment: str
    title: str
    token: str

class MessageT(TypedDict):
    link_names: bool
    text: str
    username: str
    icon_url: str
    icon_emoji: str
    blocks: list[Any]

async def async_get_service(hass: HomeAssistant, config: ConfigType, discovery_info: Union[DiscoveryInfoType, None] = ...) -> Union[SlackNotificationService, None]: ...
def _async_get_filename_from_url(url: str) -> str: ...
def _async_sanitize_channel_names(channel_list: list[str]) -> list[str]: ...

class SlackNotificationService(BaseNotificationService):
    _client: Any
    _default_channel: Any
    _hass: Any
    _icon: Any
    _username: Any
    def __init__(self, hass: HomeAssistant, client: WebClient, default_channel: str, username: Union[str, None], icon: Union[str, None]) -> None: ...
    async def _async_send_local_file_message(self, path: str, targets: list[str], message: str, title: Union[str, None]) -> None: ...
    async def _async_send_remote_file_message(self, url: str, targets: list[str], message: str, title: Union[str, None], *, username: Union[str, None] = ..., password: Union[str, None] = ...) -> None: ...
    async def _async_send_text_only_message(self, targets: list[str], message: str, title: Union[str, None], *, username: Union[str, None] = ..., icon: Union[str, None] = ..., blocks: Union[Any, None] = ...) -> None: ...
    async def async_send_message(self, message: str, **kwargs: Any) -> None: ...



import aioshelly
from .const import AIOSHELLY_DEVICE_TIMEOUT_SEC as AIOSHELLY_DEVICE_TIMEOUT_SEC, ATTR_CHANNEL as ATTR_CHANNEL, ATTR_CLICK_TYPE as ATTR_CLICK_TYPE, ATTR_DEVICE as ATTR_DEVICE, BATTERY_DEVICES_WITH_PERMANENT_CONNECTION as BATTERY_DEVICES_WITH_PERMANENT_CONNECTION, COAP as COAP, CONF_COAP_PORT as CONF_COAP_PORT, DATA_CONFIG_ENTRY as DATA_CONFIG_ENTRY, DEFAULT_COAP_PORT as DEFAULT_COAP_PORT, DEVICE as DEVICE, DOMAIN as DOMAIN, EVENT_SHELLY_CLICK as EVENT_SHELLY_CLICK, INPUTS_EVENTS_DICT as INPUTS_EVENTS_DICT, POLLING_TIMEOUT_SEC as POLLING_TIMEOUT_SEC, REST as REST, REST_SENSORS_UPDATE_INTERVAL as REST_SENSORS_UPDATE_INTERVAL, SHBTN_MODELS as SHBTN_MODELS, SLEEP_PERIOD_MULTIPLIER as SLEEP_PERIOD_MULTIPLIER, UPDATE_PERIOD_MULTIPLIER as UPDATE_PERIOD_MULTIPLIER
from .utils import get_coap_context as get_coap_context, get_device_name as get_device_name, get_device_sleep_period as get_device_sleep_period
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_DEVICE_ID as ATTR_DEVICE_ID, CONF_HOST as CONF_HOST, CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryNotReady as ConfigEntryNotReady
from homeassistant.helpers import aiohttp_client as aiohttp_client, device_registry as device_registry, update_coordinator as update_coordinator
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any, Final

PLATFORMS: Final[Any]
SLEEPING_PLATFORMS: Final[Any]
_LOGGER: Final[Any]
COAP_SCHEMA: Final[Any]
CONFIG_SCHEMA: Final[Any]

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_device_setup(hass: HomeAssistant, entry: ConfigEntry, device: aioshelly.Device) -> None: ...

class ShellyDeviceWrapper(update_coordinator.DataUpdateCoordinator):
    device_id: Any
    hass: Any
    entry: Any
    device: Any
    _async_remove_device_updates_handler: Any
    _last_input_events_count: Any
    def __init__(self, hass: HomeAssistant, entry: ConfigEntry, device: aioshelly.Device) -> None: ...
    def _async_device_updates_handler(self) -> None: ...
    async def _async_update_data(self) -> None: ...
    @property
    def model(self) -> str: ...
    @property
    def mac(self) -> str: ...
    async def async_setup(self) -> None: ...
    def shutdown(self) -> None: ...
    def _handle_ha_stop(self, _event: Event) -> None: ...

class ShellyDeviceRestWrapper(update_coordinator.DataUpdateCoordinator):
    device: Any
    def __init__(self, hass: HomeAssistant, device: aioshelly.Device) -> None: ...
    async def _async_update_data(self) -> None: ...
    @property
    def mac(self) -> str: ...

async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
def get_device_wrapper(hass: HomeAssistant, device_id: str) -> Union[ShellyDeviceWrapper, None]: ...



from . import get_device_wrapper as get_device_wrapper
from .const import ATTR_CHANNEL as ATTR_CHANNEL, ATTR_CLICK_TYPE as ATTR_CLICK_TYPE, ATTR_DEVICE as ATTR_DEVICE, DOMAIN as DOMAIN, EVENT_SHELLY_CLICK as EVENT_SHELLY_CLICK
from .utils import get_device_name as get_device_name
from homeassistant.const import ATTR_DEVICE_ID as ATTR_DEVICE_ID
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.typing import EventType as EventType
from typing import Callable

def async_describe_events(hass: HomeAssistant, async_describe_event: Callable[[str, str, Callable[[EventType], dict]], None]) -> None: ...



from .auth import setup_auth as setup_auth
from .ban import setup_bans as setup_bans
from .const import KEY_AUTHENTICATED as KEY_AUTHENTICATED, KEY_HASS as KEY_HASS, KEY_HASS_USER as KEY_HASS_USER
from .cors import setup_cors as setup_cors
from .forwarded import async_setup_forwarded as async_setup_forwarded
from .request_context import setup_request_context as setup_request_context
from .security_filter import setup_security_filter as setup_security_filter
from .static import CACHE_HEADERS as CACHE_HEADERS, CachingStaticResource as CachingStaticResource
from .view import HomeAssistantView as HomeAssistantView
from .web_runner import HomeAssistantTCPSite as HomeAssistantTCPSite
from aiohttp import web
from aiohttp.typedefs import StrOrURL as StrOrURL
from aiohttp.web_exceptions import HTTPRedirection as HTTPRedirection
from contextvars import ContextVar
from homeassistant.const import EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP, SERVER_PORT as SERVER_PORT
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant
from homeassistant.helpers import storage as storage
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.setup import async_start_setup as async_start_setup, async_when_setup_or_start as async_when_setup_or_start
from typing import Any, Final, TypedDict

DOMAIN: Final[str]
CONF_SERVER_HOST: Final[str]
CONF_SERVER_PORT: Final[str]
CONF_BASE_URL: Final[str]
CONF_SSL_CERTIFICATE: Final[str]
CONF_SSL_PEER_CERTIFICATE: Final[str]
CONF_SSL_KEY: Final[str]
CONF_CORS_ORIGINS: Final[str]
CONF_USE_X_FORWARDED_FOR: Final[str]
CONF_TRUSTED_PROXIES: Final[str]
CONF_LOGIN_ATTEMPTS_THRESHOLD: Final[str]
CONF_IP_BAN_ENABLED: Final[str]
CONF_SSL_PROFILE: Final[str]
SSL_MODERN: Final[str]
SSL_INTERMEDIATE: Final[str]
_LOGGER: Final[Any]
DEFAULT_DEVELOPMENT: Final[str]
DEFAULT_CORS: Final[list[str]]
NO_LOGIN_ATTEMPT_THRESHOLD: Final[int]
MAX_CLIENT_SIZE: Final[Any]
STORAGE_KEY: Final[Any]
STORAGE_VERSION: Final[int]
SAVE_DELAY: Final[int]
HTTP_SCHEMA: Final[Any]
CONFIG_SCHEMA: Final[Any]

class ConfData(TypedDict):
    server_host: list[str]
    server_port: int
    base_url: str
    ssl_certificate: str
    ssl_peer_certificate: str
    ssl_key: str
    cors_allowed_origins: list[str]
    use_x_forwarded_for: bool
    trusted_proxies: list[str]
    login_attempts_threshold: int
    ip_ban_enabled: bool
    ssl_profile: str

async def async_get_last_config(hass: HomeAssistant) -> Union[dict, None]: ...

class ApiConfig:
    local_ip: Any
    host: Any
    port: Any
    use_ssl: Any
    def __init__(self, local_ip: str, host: str, port: int, use_ssl: bool) -> None: ...

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...

class HomeAssistantHTTP:
    hass: Any
    ssl_certificate: Any
    ssl_peer_certificate: Any
    ssl_key: Any
    server_host: Any
    server_port: Any
    trusted_proxies: Any
    is_ban_enabled: Any
    ssl_profile: Any
    _handler: Any
    runner: Any
    site: Any
    def __init__(self, hass: HomeAssistant, ssl_certificate: Union[str, None], ssl_peer_certificate: Union[str, None], ssl_key: Union[str, None], server_host: Union[list[str], None], server_port: int, cors_origins: list[str], use_x_forwarded_for: bool, trusted_proxies: list[str], login_threshold: int, is_ban_enabled: bool, ssl_profile: str) -> None: ...
    def register_view(self, view: HomeAssistantView) -> None: ...
    def register_redirect(self, url: str, redirect_to: StrOrURL, *, redirect_exc: type[HTTPRedirection] = ...) -> None: ...
    def register_static_path(self, url_path: str, path: str, cache_headers: bool = ...) -> Union[web.FileResponse, None]: ...
    async def start(self) -> None: ...
    async def stop(self) -> None: ...

async def start_http_server_and_save_config(hass: HomeAssistant, conf: dict, server: HomeAssistantHTTP) -> None: ...

current_request: ContextVar[Union[web.Request, None]]



import voluptuous as vol
from . import MULTI_FACTOR_AUTH_MODULES as MULTI_FACTOR_AUTH_MODULES, MULTI_FACTOR_AUTH_MODULE_SCHEMA as MULTI_FACTOR_AUTH_MODULE_SCHEMA, MultiFactorAuthModule as MultiFactorAuthModule, SetupFlow as SetupFlow
from homeassistant.auth.models import User as User
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.data_entry_flow import FlowResult as FlowResult
from typing import Any

REQUIREMENTS: Any
CONFIG_SCHEMA: Any
STORAGE_VERSION: int
STORAGE_KEY: str
STORAGE_USERS: str
STORAGE_USER_ID: str
STORAGE_OTA_SECRET: str
INPUT_FIELD_CODE: str
DUMMY_SECRET: str

def _generate_qr_code(data: str) -> str: ...
def _generate_secret_and_qr_code(username: str) -> tuple[str, str, str]: ...

class TotpAuthModule(MultiFactorAuthModule):
    DEFAULT_TITLE: str
    MAX_RETRY_TIME: int
    _users: Any
    _user_store: Any
    _init_lock: Any
    def __init__(self, hass: HomeAssistant, config: dict[str, Any]) -> None: ...
    @property
    def input_schema(self) -> vol.Schema: ...
    async def _async_load(self) -> None: ...
    async def _async_save(self) -> None: ...
    def _add_ota_secret(self, user_id: str, secret: Union[str, None] = ...) -> str: ...
    async def async_setup_flow(self, user_id: str) -> SetupFlow: ...
    async def async_setup_user(self, user_id: str, setup_data: Any) -> str: ...
    async def async_depose_user(self, user_id: str) -> None: ...
    async def async_is_user_setup(self, user_id: str) -> bool: ...
    async def async_validate(self, user_id: str, user_input: dict[str, Any]) -> bool: ...
    def _validate_2fa(self, user_id: str, code: str) -> bool: ...

class TotpSetupFlow(SetupFlow):
    _auth_module: Any
    _user: Any
    _ota_secret: Any
    _url: Any
    _image: Any
    def __init__(self, auth_module: TotpAuthModule, setup_schema: vol.Schema, user: User) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...



from . import ValveControllerEntity as ValveControllerEntity
from .const import API_VALVE_STATUS as API_VALVE_STATUS, CONF_UID as CONF_UID, DATA_CLIENT as DATA_CLIENT, DATA_COORDINATOR as DATA_COORDINATOR, DATA_PAIRED_SENSOR_MANAGER as DATA_PAIRED_SENSOR_MANAGER, DOMAIN as DOMAIN, LOGGER as LOGGER
from aioguardian import Client as Client
from homeassistant.components.switch import SwitchEntity as SwitchEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_FILENAME as CONF_FILENAME, CONF_PORT as CONF_PORT, CONF_URL as CONF_URL
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import entity_platform as entity_platform
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any

ATTR_AVG_CURRENT: str
ATTR_INST_CURRENT: str
ATTR_INST_CURRENT_DDT: str
ATTR_TRAVEL_COUNT: str
SERVICE_DISABLE_AP: str
SERVICE_ENABLE_AP: str
SERVICE_PAIR_SENSOR: str
SERVICE_REBOOT: str
SERVICE_RESET_VALVE_DIAGNOSTICS: str
SERVICE_UNPAIR_SENSOR: str
SERVICE_UPGRADE_FIRMWARE: str

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class ValveControllerSwitch(ValveControllerEntity, SwitchEntity):
    _attr_is_on: bool
    _client: Any
    def __init__(self, entry: ConfigEntry, client: Client, coordinators: dict[str, DataUpdateCoordinator]) -> None: ...
    async def _async_continue_entity_setup(self) -> None: ...
    _attr_available: Any
    def _async_update_from_latest_data(self) -> None: ...
    async def async_disable_ap(self) -> None: ...
    async def async_enable_ap(self) -> None: ...
    async def async_pair_sensor(self, uid: str) -> None: ...
    async def async_reboot(self) -> None: ...
    async def async_reset_valve_diagnostics(self) -> None: ...
    async def async_unpair_sensor(self, uid: str) -> None: ...
    async def async_upgrade_firmware(self, url: str, port: int, filename: str) -> None: ...
    async def async_turn_off(self, **kwargs: dict[str, Any]) -> None: ...
    async def async_turn_on(self, **kwargs: dict[str, Any]) -> None: ...



from homeassistant.components.group import GroupIntegrationRegistry as GroupIntegrationRegistry
from homeassistant.const import STATE_ALARM_ARMED_AWAY as STATE_ALARM_ARMED_AWAY, STATE_ALARM_ARMED_CUSTOM_BYPASS as STATE_ALARM_ARMED_CUSTOM_BYPASS, STATE_ALARM_ARMED_HOME as STATE_ALARM_ARMED_HOME, STATE_ALARM_ARMED_NIGHT as STATE_ALARM_ARMED_NIGHT, STATE_ALARM_ARMED_VACATION as STATE_ALARM_ARMED_VACATION, STATE_ALARM_TRIGGERED as STATE_ALARM_TRIGGERED, STATE_OFF as STATE_OFF
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback

def async_describe_on_off_states(hass: HomeAssistant, registry: GroupIntegrationRegistry) -> None: ...



from typing import Any, Final, Tuple, TypedDict

ATTR_DEVICES: Final[str]
ATTR_GATEWAY_ID: Final[str]
CONF_BAUD_RATE: Final[str]
CONF_DEVICE: Final[str]
CONF_GATEWAYS: Final[str]
CONF_NODES: Final[str]
CONF_PERSISTENCE: Final[str]
CONF_PERSISTENCE_FILE: Final[str]
CONF_RETAIN: Final[str]
CONF_TCP_PORT: Final[str]
CONF_TOPIC_IN_PREFIX: Final[str]
CONF_TOPIC_OUT_PREFIX: Final[str]
CONF_VERSION: Final[str]
CONF_GATEWAY_TYPE: Final[str]
ConfGatewayType: Any
CONF_GATEWAY_TYPE_SERIAL: ConfGatewayType
CONF_GATEWAY_TYPE_TCP: ConfGatewayType
CONF_GATEWAY_TYPE_MQTT: ConfGatewayType
CONF_GATEWAY_TYPE_ALL: list[str]
DOMAIN: Final[str]
MYSENSORS_GATEWAY_START_TASK: str
MYSENSORS_GATEWAYS: Final[str]
PLATFORM: Final[str]
SCHEMA: Final[str]
CHILD_CALLBACK: str
NODE_CALLBACK: str
MYSENSORS_DISCOVERY: str
MYSENSORS_ON_UNLOAD: str
TYPE: Final[str]
UPDATE_DELAY: float

class DiscoveryInfo(TypedDict):
    devices: list[DevId]
    name: str
    gateway_id: GatewayId

SERVICE_SEND_IR_CODE: Final[str]
SensorType = str
ValueType = str
GatewayId = str
DevId = Tuple[GatewayId, int, int, int]
BINARY_SENSOR_TYPES: dict[SensorType, set[ValueType]]
CLIMATE_TYPES: dict[SensorType, set[ValueType]]
COVER_TYPES: dict[SensorType, set[ValueType]]
DEVICE_TRACKER_TYPES: dict[SensorType, set[ValueType]]
LIGHT_TYPES: dict[SensorType, set[ValueType]]
NOTIFY_TYPES: dict[SensorType, set[ValueType]]
SENSOR_TYPES: dict[SensorType, set[ValueType]]
SWITCH_TYPES: dict[SensorType, set[ValueType]]
PLATFORM_TYPES: dict[str, dict[SensorType, set[ValueType]]]
FLAT_PLATFORM_TYPES: dict[tuple[str, SensorType], set[ValueType]]
TYPE_TO_PLATFORMS: dict[SensorType, list[str]]
PLATFORMS_WITH_ENTRY_SUPPORT: Any



from .const import DOMAIN as DOMAIN, KNX_ADDRESS as KNX_ADDRESS
from .knx_entity import KnxEntity as KnxEntity
from .schema import FanSchema as FanSchema
from homeassistant.components.fan import FanEntity as FanEntity, SUPPORT_OSCILLATE as SUPPORT_OSCILLATE, SUPPORT_SET_SPEED as SUPPORT_SET_SPEED
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from homeassistant.util.percentage import int_states_in_range as int_states_in_range, percentage_to_ranged_value as percentage_to_ranged_value, ranged_value_to_percentage as ranged_value_to_percentage
from typing import Any, Final
from xknx import XKNX as XKNX
from xknx.devices import Fan as XknxFan

DEFAULT_PERCENTAGE: Final[int]

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class KNXFan(KnxEntity, FanEntity):
    _device: XknxFan
    _step_range: Any
    _attr_unique_id: Any
    def __init__(self, xknx: XKNX, config: ConfigType) -> None: ...
    async def async_set_percentage(self, percentage: int) -> None: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def percentage(self) -> Union[int, None]: ...
    @property
    def speed_count(self) -> int: ...
    async def async_turn_on(self, speed: Union[str, None] = ..., percentage: Union[int, None] = ..., preset_mode: Union[str, None] = ..., **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def async_oscillate(self, oscillating: bool) -> None: ...
    @property
    def oscillating(self) -> Union[bool, None]: ...



from typing import Any, Final

DOMAIN: Final[str]
ATTRIBUTION: Final[str]
ATTR_TRACKINGS: Final[str]
BASE: Final[str]
CONF_SLUG: Final[str]
CONF_TITLE: Final[str]
CONF_TRACKING_NUMBER: Final[str]
DEFAULT_NAME: Final[str]
UPDATE_TOPIC: Final[Any]
ICON: Final[str]
MIN_TIME_BETWEEN_UPDATES: Final[Any]
SERVICE_ADD_TRACKING: Final[str]
SERVICE_REMOVE_TRACKING: Final[str]
ADD_TRACKING_SERVICE_SCHEMA: Final[Any]
REMOVE_TRACKING_SERVICE_SCHEMA: Final[Any]



from .config_entry import async_setup_entry as async_setup_entry, async_unload_entry as async_unload_entry
from .const import ATTR_ATTRIBUTES as ATTR_ATTRIBUTES, ATTR_BATTERY as ATTR_BATTERY, ATTR_DEV_ID as ATTR_DEV_ID, ATTR_GPS as ATTR_GPS, ATTR_HOST_NAME as ATTR_HOST_NAME, ATTR_LOCATION_NAME as ATTR_LOCATION_NAME, ATTR_MAC as ATTR_MAC, ATTR_SOURCE_TYPE as ATTR_SOURCE_TYPE, CONF_CONSIDER_HOME as CONF_CONSIDER_HOME, CONF_NEW_DEVICE_DEFAULTS as CONF_NEW_DEVICE_DEFAULTS, CONF_SCAN_INTERVAL as CONF_SCAN_INTERVAL, CONF_TRACK_NEW as CONF_TRACK_NEW, DOMAIN as DOMAIN, SOURCE_TYPE_BLUETOOTH as SOURCE_TYPE_BLUETOOTH, SOURCE_TYPE_BLUETOOTH_LE as SOURCE_TYPE_BLUETOOTH_LE, SOURCE_TYPE_GPS as SOURCE_TYPE_GPS, SOURCE_TYPE_ROUTER as SOURCE_TYPE_ROUTER
from .legacy import DeviceScanner as DeviceScanner, PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE, SERVICE_SEE as SERVICE_SEE, SERVICE_SEE_PAYLOAD_SCHEMA as SERVICE_SEE_PAYLOAD_SCHEMA, SOURCE_TYPES as SOURCE_TYPES, see as see
from homeassistant.const import ATTR_GPS_ACCURACY as ATTR_GPS_ACCURACY, STATE_HOME as STATE_HOME
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.loader import bind_hass as bind_hass

def is_on(hass: HomeAssistant, entity_id: str) -> bool: ...
async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...



from .const import DOMAIN as DOMAIN
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_HOST as CONF_HOST
from homeassistant.core import HomeAssistant as HomeAssistant
from typing import Final

PLATFORMS: Final[list[str]]

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...



from .dispatcher import async_dispatcher_connect as async_dispatcher_connect, async_dispatcher_send as async_dispatcher_send
from .typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from homeassistant import core as core, setup as setup
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE
from homeassistant.loader import bind_hass as bind_hass
from typing import Any, Callable, TypedDict

SIGNAL_PLATFORM_DISCOVERED: str
EVENT_LOAD_PLATFORM: str
ATTR_PLATFORM: str
ATTR_DISCOVERED: str

class DiscoveryDict(TypedDict):
    service: str
    platform: Union[str, None]
    discovered: Union[DiscoveryInfoType, None]

def async_listen(hass: core.HomeAssistant, service: str, callback: CALLBACK_TYPE) -> None: ...
def discover(hass: core.HomeAssistant, service: str, discovered: DiscoveryInfoType, component: str, hass_config: ConfigType) -> None: ...
async def async_discover(hass: core.HomeAssistant, service: str, discovered: Union[DiscoveryInfoType, None], component: Union[str, None], hass_config: ConfigType) -> None: ...
def async_listen_platform(hass: core.HomeAssistant, component: str, callback: Callable[[str, Union[dict[str, Any], None]], Any]) -> None: ...
def load_platform(hass: core.HomeAssistant, component: str, platform: str, discovered: DiscoveryInfoType, hass_config: ConfigType) -> None: ...
async def async_load_platform(hass: core.HomeAssistant, component: str, platform: str, discovered: DiscoveryInfoType, hass_config: ConfigType) -> None: ...



from .config import AutomationConfig as AutomationConfig, async_validate_config_item as async_validate_config_item
from .const import CONF_ACTION as CONF_ACTION, CONF_INITIAL_STATE as CONF_INITIAL_STATE, CONF_TRACE as CONF_TRACE, CONF_TRIGGER as CONF_TRIGGER, CONF_TRIGGER_VARIABLES as CONF_TRIGGER_VARIABLES, DEFAULT_INITIAL_STATE as DEFAULT_INITIAL_STATE, DOMAIN as DOMAIN, LOGGER as LOGGER
from .helpers import async_get_blueprints as async_get_blueprints
from .trace import trace_automation as trace_automation
from homeassistant.components import blueprint as blueprint
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, ATTR_MODE as ATTR_MODE, ATTR_NAME as ATTR_NAME, CONF_ALIAS as CONF_ALIAS, CONF_CONDITION as CONF_CONDITION, CONF_DEVICE_ID as CONF_DEVICE_ID, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_ID as CONF_ID, CONF_MODE as CONF_MODE, CONF_PLATFORM as CONF_PLATFORM, CONF_VARIABLES as CONF_VARIABLES, CONF_ZONE as CONF_ZONE, EVENT_HOMEASSISTANT_STARTED as EVENT_HOMEASSISTANT_STARTED, SERVICE_RELOAD as SERVICE_RELOAD, SERVICE_TOGGLE as SERVICE_TOGGLE, SERVICE_TURN_OFF as SERVICE_TURN_OFF, SERVICE_TURN_ON as SERVICE_TURN_ON, STATE_ON as STATE_ON
from homeassistant.core import Context as Context, CoreState as CoreState, HomeAssistant as HomeAssistant, callback as callback, split_entity_id as split_entity_id
from homeassistant.exceptions import ConditionError as ConditionError, ConditionErrorContainer as ConditionErrorContainer, ConditionErrorIndex as ConditionErrorIndex, HomeAssistantError as HomeAssistantError
from homeassistant.helpers import condition as condition, extract_domain_configs as extract_domain_configs, template as template
from homeassistant.helpers.entity import ToggleEntity as ToggleEntity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from homeassistant.helpers.restore_state import RestoreEntity as RestoreEntity
from homeassistant.helpers.script import ATTR_CUR as ATTR_CUR, ATTR_MAX as ATTR_MAX, CONF_MAX as CONF_MAX, CONF_MAX_EXCEEDED as CONF_MAX_EXCEEDED, Script as Script
from homeassistant.helpers.script_variables import ScriptVariables as ScriptVariables
from homeassistant.helpers.service import ReloadServiceHelper as ReloadServiceHelper, async_register_admin_service as async_register_admin_service
from homeassistant.helpers.trace import TraceElement as TraceElement, script_execution_set as script_execution_set, trace_append_element as trace_append_element, trace_get as trace_get, trace_path as trace_path
from homeassistant.helpers.trigger import async_initialize_triggers as async_initialize_triggers
from homeassistant.helpers.typing import TemplateVarsType as TemplateVarsType
from homeassistant.loader import bind_hass as bind_hass
from homeassistant.util.dt import parse_datetime as parse_datetime
from typing import Any, Awaitable, Callable

ENTITY_ID_FORMAT: Any
CONF_SKIP_CONDITION: str
CONF_STOP_ACTIONS: str
DEFAULT_STOP_ACTIONS: bool
EVENT_AUTOMATION_RELOADED: str
EVENT_AUTOMATION_TRIGGERED: str
ATTR_LAST_TRIGGERED: str
ATTR_SOURCE: str
ATTR_VARIABLES: str
SERVICE_TRIGGER: str
_LOGGER: Any
AutomationActionType = Callable[[HomeAssistant, TemplateVarsType], Awaitable[None]]

def is_on(hass, entity_id): ...
def automations_with_entity(hass: HomeAssistant, entity_id: str) -> list[str]: ...
def entities_in_automation(hass: HomeAssistant, entity_id: str) -> list[str]: ...
def automations_with_device(hass: HomeAssistant, device_id: str) -> list[str]: ...
def devices_in_automation(hass: HomeAssistant, entity_id: str) -> list[str]: ...
def automations_with_area(hass: HomeAssistant, area_id: str) -> list[str]: ...
def areas_in_automation(hass: HomeAssistant, entity_id: str) -> list[str]: ...
async def async_setup(hass, config): ...

class AutomationEntity(ToggleEntity, RestoreEntity):
    _attr_should_poll: bool
    _attr_name: Any
    _trigger_config: Any
    _async_detach_triggers: Any
    _cond_func: Any
    action_script: Any
    _initial_state: Any
    _is_enabled: bool
    _referenced_entities: Any
    _referenced_devices: Any
    _logger: Any
    _variables: Any
    _trigger_variables: Any
    _raw_config: Any
    _blueprint_inputs: Any
    _trace_config: Any
    _attr_unique_id: Any
    def __init__(self, automation_id, name, trigger_config, cond_func, action_script, initial_state, variables, trigger_variables, raw_config, blueprint_inputs, trace_config) -> None: ...
    @property
    def extra_state_attributes(self): ...
    @property
    def is_on(self) -> bool: ...
    @property
    def referenced_areas(self): ...
    @property
    def referenced_devices(self): ...
    @property
    def referenced_entities(self): ...
    async def async_added_to_hass(self) -> None: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def async_trigger(self, run_variables, context: Any | None = ..., skip_condition: bool = ...) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...
    async def async_enable(self) -> None: ...
    async def async_disable(self, stop_actions=...) -> None: ...
    async def _async_attach_triggers(self, home_assistant_start: bool) -> Union[Callable[[], None], None]: ...

async def _async_process_config(hass: HomeAssistant, config: dict[str, Any], component: EntityComponent) -> bool: ...
async def _async_process_if(hass, name, config, p_config): ...
def _trigger_extract_device(trigger_conf: dict) -> Union[str, None]: ...
def _trigger_extract_entities(trigger_conf: dict) -> list[str]: ...



from aiohttp.web import Application as Application, Request as Request, StreamResponse as StreamResponse
from collections.abc import Callable as Callable
from contextvars import ContextVar as ContextVar
from homeassistant.core import callback as callback

def setup_request_context(app: Application, context: ContextVar[Union[Request, None]]) -> None: ...



from .const import DOMAIN as DOMAIN
from homeassistant import config_entries as config_entries, exceptions as exceptions
from homeassistant.const import CONF_HOST as CONF_HOST
from homeassistant.data_entry_flow import FlowResult as FlowResult
from typing import Any, Final

_LOGGER: Final[Any]

def host_valid(host: str) -> bool: ...

class DuneHDConfigFlow(config_entries.ConfigFlow):
    VERSION: int
    async def init_device(self, host: str) -> None: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_import(self, user_input: Union[dict[str, str], None] = ...) -> FlowResult: ...
    def host_already_configured(self, host: str) -> bool: ...

class CannotConnect(exceptions.HomeAssistantError): ...
class AlreadyConfigured(exceptions.HomeAssistantError): ...



from .const import CONTROLLER_MODES as CONTROLLER_MODES, CURRENT_HVAC_ACTIONS as CURRENT_HVAC_ACTIONS, DOMAIN as DOMAIN, PRESET_MODES as PRESET_MODES
from .knx_entity import KnxEntity as KnxEntity
from .schema import ClimateSchema as ClimateSchema
from homeassistant.components.climate import ClimateEntity as ClimateEntity
from homeassistant.components.climate.const import CURRENT_HVAC_IDLE as CURRENT_HVAC_IDLE, CURRENT_HVAC_OFF as CURRENT_HVAC_OFF, HVAC_MODE_HEAT as HVAC_MODE_HEAT, HVAC_MODE_OFF as HVAC_MODE_OFF, PRESET_AWAY as PRESET_AWAY, SUPPORT_PRESET_MODE as SUPPORT_PRESET_MODE, SUPPORT_TARGET_TEMPERATURE as SUPPORT_TARGET_TEMPERATURE
from homeassistant.const import ATTR_TEMPERATURE as ATTR_TEMPERATURE, CONF_NAME as CONF_NAME, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any
from xknx import XKNX as XKNX
from xknx.devices import Climate as XknxClimate

ATTR_COMMAND_VALUE: str
CONTROLLER_MODES_INV: Any
PRESET_MODES_INV: Any

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...
def _async_migrate_unique_id(hass: HomeAssistant, platform_config: list[ConfigType]) -> None: ...
def _create_climate(xknx: XKNX, config: ConfigType) -> XknxClimate: ...

class KNXClimate(KnxEntity, ClimateEntity):
    _device: XknxClimate
    _attr_temperature_unit: Any
    _attr_supported_features: Any
    _attr_target_temperature_step: Any
    _attr_unique_id: Any
    def __init__(self, xknx: XKNX, config: ConfigType) -> None: ...
    async def async_update(self) -> None: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature(self) -> Union[float, None]: ...
    @property
    def min_temp(self) -> float: ...
    @property
    def max_temp(self) -> float: ...
    async def async_set_temperature(self, **kwargs: Any) -> None: ...
    @property
    def hvac_mode(self) -> str: ...
    @property
    def hvac_modes(self) -> list[str]: ...
    @property
    def hvac_action(self) -> Union[str, None]: ...
    async def async_set_hvac_mode(self, hvac_mode: str) -> None: ...
    @property
    def preset_mode(self) -> Union[str, None]: ...
    @property
    def preset_modes(self) -> Union[list[str], None]: ...
    async def async_set_preset_mode(self, preset_mode: str) -> None: ...
    @property
    def extra_state_attributes(self) -> Union[dict[str, Any], None]: ...
    async def async_added_to_hass(self) -> None: ...
    async def async_will_remove_from_hass(self) -> None: ...



from . import EsphomeEntity as EsphomeEntity, EsphomeEnumMapper as EsphomeEnumMapper, esphome_state_property as esphome_state_property, platform_async_setup_entry as platform_async_setup_entry
from aioesphomeapi import FanDirection, FanInfo, FanState
from homeassistant.components.fan import DIRECTION_FORWARD as DIRECTION_FORWARD, DIRECTION_REVERSE as DIRECTION_REVERSE, FanEntity as FanEntity, SUPPORT_DIRECTION as SUPPORT_DIRECTION, SUPPORT_OSCILLATE as SUPPORT_OSCILLATE, SUPPORT_SET_SPEED as SUPPORT_SET_SPEED
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.percentage import ordered_list_item_to_percentage as ordered_list_item_to_percentage, percentage_to_ordered_list_item as percentage_to_ordered_list_item, percentage_to_ranged_value as percentage_to_ranged_value, ranged_value_to_percentage as ranged_value_to_percentage
from typing import Any

ORDERED_NAMED_FAN_SPEEDS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

_FAN_DIRECTIONS: EsphomeEnumMapper[FanDirection, str]

class EsphomeFan(EsphomeEntity[FanInfo, FanState], FanEntity):
    @property
    def _supports_speed_levels(self) -> bool: ...
    async def async_set_percentage(self, percentage: Union[int, None]) -> None: ...
    async def async_turn_on(self, speed: Union[str, None] = ..., percentage: Union[int, None] = ..., preset_mode: Union[str, None] = ..., **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def async_oscillate(self, oscillating: bool) -> None: ...
    async def async_set_direction(self, direction: str) -> None: ...
    def is_on(self) -> Union[bool, None]: ...
    def percentage(self) -> Union[int, None]: ...
    @property
    def speed_count(self) -> int: ...
    def oscillating(self) -> Union[bool, None]: ...
    def current_direction(self) -> Union[str, None]: ...
    @property
    def supported_features(self) -> int: ...



from . import AmbientStation as AmbientStation, AmbientWeatherEntity as AmbientWeatherEntity, SENSOR_TYPES as SENSOR_TYPES, TYPE_SOLARRADIATION as TYPE_SOLARRADIATION, TYPE_SOLARRADIATION_LX as TYPE_SOLARRADIATION_LX
from .const import ATTR_LAST_DATA as ATTR_LAST_DATA, ATTR_MONITORED_CONDITIONS as ATTR_MONITORED_CONDITIONS, DATA_CLIENT as DATA_CLIENT, DOMAIN as DOMAIN
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_NAME as ATTR_NAME
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class AmbientWeatherSensor(AmbientWeatherEntity, SensorEntity):
    _attr_unit_of_measurement: Any
    def __init__(self, ambient: AmbientStation, mac_address: str, station_name: str, sensor_type: str, sensor_name: str, device_class: Union[str, None], unit: Union[str, None]) -> None: ...
    _attr_state: Any
    def update_from_latest_data(self) -> None: ...



from . import Router as Router
from .const import ATTR_UNIQUE_ID as ATTR_UNIQUE_ID, DOMAIN as DOMAIN
from homeassistant.components.notify import ATTR_TARGET as ATTR_TARGET, BaseNotificationService as BaseNotificationService
from homeassistant.const import CONF_RECIPIENT as CONF_RECIPIENT
from homeassistant.core import HomeAssistant as HomeAssistant
from typing import Any

_LOGGER: Any

async def async_get_service(hass: HomeAssistant, config: dict[str, Any], discovery_info: Union[dict[str, Any], None] = ...) -> Union[HuaweiLteSmsNotificationService, None]: ...

class HuaweiLteSmsNotificationService(BaseNotificationService):
    router: Router
    default_targets: list[str]
    def send_message(self, message: str = ..., **kwargs: Any) -> None: ...
    def __init__(self, router, default_targets) -> None: ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...



from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_LATITUDE as ATTR_LATITUDE, ATTR_LONGITUDE as ATTR_LONGITUDE
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.config_validation import PLATFORM_SCHEMA as PLATFORM_SCHEMA, PLATFORM_SCHEMA_BASE as PLATFORM_SCHEMA_BASE
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_component import EntityComponent as EntityComponent
from typing import Any

_LOGGER: Any
ATTR_DISTANCE: str
ATTR_SOURCE: str
DOMAIN: str
ENTITY_ID_FORMAT: Any
SCAN_INTERVAL: Any

async def async_setup(hass, config): ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class GeolocationEvent(Entity):
    @property
    def state(self): ...
    @property
    def source(self) -> str: ...
    @property
    def distance(self) -> Union[float, None]: ...
    @property
    def latitude(self) -> Union[float, None]: ...
    @property
    def longitude(self) -> Union[float, None]: ...
    @property
    def state_attributes(self): ...



from .const import CONF_DEVICE_PASSWORD as CONF_DEVICE_PASSWORD, CONF_PHONE_ID as CONF_PHONE_ID, DATA_DEVICE as DATA_DEVICE, DATA_DISCOVERY as DATA_DISCOVERY, DOMAIN as DOMAIN, MAX_UPDATE_INTERVAL_SEC as MAX_UPDATE_INTERVAL_SEC, SIGNAL_DEVICE_ADD as SIGNAL_DEVICE_ADD
from .utils import async_start_bridge as async_start_bridge, async_stop_bridge as async_stop_bridge
from aioswitcher.device import SwitcherBase as SwitcherBase
from homeassistant.config_entries import ConfigEntry as ConfigEntry, SOURCE_IMPORT as SOURCE_IMPORT
from homeassistant.const import CONF_DEVICE_ID as CONF_DEVICE_ID, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import device_registry as device_registry, update_coordinator as update_coordinator
from homeassistant.helpers.dispatcher import async_dispatcher_send as async_dispatcher_send
from typing import Any

PLATFORMS: Any
_LOGGER: Any
CCONFIG_SCHEMA: Any

async def async_setup(hass: HomeAssistant, config: dict) -> bool: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...

class SwitcherDeviceWrapper(update_coordinator.DataUpdateCoordinator):
    hass: Any
    entry: Any
    data: Any
    def __init__(self, hass: HomeAssistant, entry: ConfigEntry, device: SwitcherBase) -> None: ...
    async def _async_update_data(self) -> None: ...
    @property
    def model(self) -> str: ...
    @property
    def device_id(self) -> str: ...
    @property
    def mac_address(self) -> str: ...
    async def async_setup(self) -> None: ...

async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...



from .const import ATTR_DEVICES as ATTR_DEVICES, CONF_BAUD_RATE as CONF_BAUD_RATE, CONF_DEVICE as CONF_DEVICE, CONF_GATEWAYS as CONF_GATEWAYS, CONF_NODES as CONF_NODES, CONF_PERSISTENCE as CONF_PERSISTENCE, CONF_PERSISTENCE_FILE as CONF_PERSISTENCE_FILE, CONF_RETAIN as CONF_RETAIN, CONF_TCP_PORT as CONF_TCP_PORT, CONF_TOPIC_IN_PREFIX as CONF_TOPIC_IN_PREFIX, CONF_TOPIC_OUT_PREFIX as CONF_TOPIC_OUT_PREFIX, CONF_VERSION as CONF_VERSION, DOMAIN as DOMAIN, DevId as DevId, DiscoveryInfo as DiscoveryInfo, MYSENSORS_DISCOVERY as MYSENSORS_DISCOVERY, MYSENSORS_GATEWAYS as MYSENSORS_GATEWAYS, MYSENSORS_ON_UNLOAD as MYSENSORS_ON_UNLOAD, PLATFORMS_WITH_ENTRY_SUPPORT as PLATFORMS_WITH_ENTRY_SUPPORT, SensorType as SensorType
from .device import MySensorsDevice as MySensorsDevice, get_mysensors_devices as get_mysensors_devices
from .gateway import finish_setup as finish_setup, get_mysensors_gateway as get_mysensors_gateway, gw_stop as gw_stop, setup_gateway as setup_gateway
from .helpers import on_unload as on_unload
from homeassistant import config_entries as config_entries
from homeassistant.components.mqtt import valid_publish_topic as valid_publish_topic, valid_subscribe_topic as valid_subscribe_topic
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_OPTIMISTIC as CONF_OPTIMISTIC
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.discovery import async_load_platform as async_load_platform
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.typing import ConfigType as ConfigType
from mysensors import BaseAsyncGateway as BaseAsyncGateway
from typing import Any, Callable

_LOGGER: Any
CONF_DEBUG: str
CONF_NODE_NAME: str
DATA_HASS_CONFIG: str
DEFAULT_BAUD_RATE: int
DEFAULT_TCP_PORT: int
DEFAULT_VERSION: str

def set_default_persistence_file(value: dict) -> dict: ...
def has_all_unique_files(value: list[dict]) -> list[dict]: ...
def is_persistence_file(value: str) -> str: ...
def deprecated(key: str) -> Callable[[dict], dict]: ...

NODE_SCHEMA: Any
GATEWAY_SCHEMA: Any
CONFIG_SCHEMA: Any

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool: ...
def setup_mysensors_platform(hass: HomeAssistant, domain: str, discovery_info: DiscoveryInfo, device_class: Union[type[MySensorsDevice], dict[SensorType, type[MySensorsDevice]]], device_args: Union[None, tuple] = ..., async_add_entities: Union[Callable, None] = ...) -> Union[list[MySensorsDevice], None]: ...



from homeassistant.components.binary_sensor import DEVICE_CLASS_SAFETY as DEVICE_CLASS_SAFETY
from homeassistant.components.sensor import ATTR_STATE_CLASS as ATTR_STATE_CLASS, STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT
from homeassistant.const import ATTR_DEVICE_CLASS as ATTR_DEVICE_CLASS, ATTR_ICON as ATTR_ICON, ATTR_NAME as ATTR_NAME, ATTR_UNIT_OF_MEASUREMENT as ATTR_UNIT_OF_MEASUREMENT, DATA_MEGABYTES as DATA_MEGABYTES, DATA_RATE_KILOBYTES_PER_SECOND as DATA_RATE_KILOBYTES_PER_SECOND, DATA_TERABYTES as DATA_TERABYTES, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, DEVICE_CLASS_TIMESTAMP as DEVICE_CLASS_TIMESTAMP, PERCENTAGE as PERCENTAGE
from typing import Any, Final, TypedDict

class EntityInfo(TypedDict):
    name: str
    unit_of_measurement: Union[str, None]
    icon: Union[str, None]
    device_class: Union[str, None]
    state_class: Union[str, None]
    enable: bool

DOMAIN: str
PLATFORMS: Any
COORDINATOR_CAMERAS: str
COORDINATOR_CENTRAL: str
COORDINATOR_SWITCHES: str
SYSTEM_LOADED: str
EXCEPTION_DETAILS: str
EXCEPTION_UNKNOWN: str
SYNO_API: str
UNDO_UPDATE_LISTENER: str
CONF_SERIAL: str
CONF_VOLUMES: str
CONF_DEVICE_TOKEN: str
DEFAULT_USE_SSL: bool
DEFAULT_VERIFY_SSL: bool
DEFAULT_PORT: int
DEFAULT_PORT_SSL: int
DEFAULT_SCAN_INTERVAL: int
DEFAULT_TIMEOUT: int
ENTITY_UNIT_LOAD: str
ENTITY_ENABLE: Final[str]
SERVICE_REBOOT: str
SERVICE_SHUTDOWN: str
SERVICES: Any
UPGRADE_BINARY_SENSORS: dict[str, EntityInfo]
SECURITY_BINARY_SENSORS: dict[str, EntityInfo]
STORAGE_DISK_BINARY_SENSORS: dict[str, EntityInfo]
UTILISATION_SENSORS: dict[str, EntityInfo]
STORAGE_VOL_SENSORS: dict[str, EntityInfo]
STORAGE_DISK_SENSORS: dict[str, EntityInfo]
INFORMATION_SENSORS: dict[str, EntityInfo]
SURVEILLANCE_SWITCH: dict[str, EntityInfo]
TEMP_SENSORS_KEYS: Any



from .const import DOMAIN as DOMAIN
from .devolo_multi_level_switch import DevoloMultiLevelSwitchDeviceEntity as DevoloMultiLevelSwitchDeviceEntity
from devolo_home_control_api.devices.zwave import Zwave as Zwave
from devolo_home_control_api.homecontrol import HomeControl as HomeControl
from homeassistant.components.cover import CoverEntity as CoverEntity, DEVICE_CLASS_BLIND as DEVICE_CLASS_BLIND, SUPPORT_CLOSE as SUPPORT_CLOSE, SUPPORT_OPEN as SUPPORT_OPEN, SUPPORT_SET_POSITION as SUPPORT_SET_POSITION
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class DevoloCoverDeviceEntity(DevoloMultiLevelSwitchDeviceEntity, CoverEntity):
    _attr_device_class: Any
    _attr_supported_features: Any
    def __init__(self, homecontrol: HomeControl, device_instance: Zwave, element_uid: str) -> None: ...
    @property
    def current_cover_position(self) -> int: ...
    @property
    def is_closed(self) -> bool: ...
    def open_cover(self, **kwargs: Any) -> None: ...
    def close_cover(self, **kwargs: Any) -> None: ...
    def set_cover_position(self, **kwargs: Any) -> None: ...



from homeassistant.const import ATTR_FRIENDLY_NAME as ATTR_FRIENDLY_NAME, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_EVENT as CONF_EVENT, CONF_PLATFORM as CONF_PLATFORM, CONF_ZONE as CONF_ZONE
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, HassJob as HassJob, callback as callback
from homeassistant.helpers import condition as condition, location as location
from homeassistant.helpers.event import async_track_state_change_event as async_track_state_change_event
from typing import Any

EVENT_ENTER: str
EVENT_LEAVE: str
DEFAULT_EVENT = EVENT_ENTER
_EVENT_DESCRIPTION: Any
TRIGGER_SCHEMA: Any

async def async_attach_trigger(hass, config, action, automation_info, *, platform_type: str = ...) -> CALLBACK_TYPE: ...



from homeassistant.components import websocket_api as websocket_api
from homeassistant.components.websocket_api.connection import ActiveConnection as ActiveConnection
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.storage import Store as Store
from typing import Any, Callable

DATA_STORAGE: str
STORAGE_VERSION_USER_DATA: int

async def async_setup_frontend_storage(hass: HomeAssistant) -> None: ...
def with_store(orig_func: Callable) -> Callable: ...
async def websocket_set_user_data(hass: HomeAssistant, connection: ActiveConnection, msg: dict, store: Store, data: dict[str, Any]) -> None: ...
async def websocket_get_user_data(hass: HomeAssistant, connection: ActiveConnection, msg: dict, store: Store, data: dict[str, Any]) -> None: ...



from .const import BPUP_SUBS as BPUP_SUBS, DOMAIN as DOMAIN, HUB as HUB
from .entity import BondEntity as BondEntity
from .utils import BondDevice as BondDevice, BondHub as BondHub
from bond_api import BPUPSubscriptions as BPUPSubscriptions
from homeassistant.components.fan import DIRECTION_FORWARD as DIRECTION_FORWARD, DIRECTION_REVERSE as DIRECTION_REVERSE, FanEntity as FanEntity, SUPPORT_DIRECTION as SUPPORT_DIRECTION, SUPPORT_SET_SPEED as SUPPORT_SET_SPEED
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import Entity as Entity
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.util.percentage import int_states_in_range as int_states_in_range, percentage_to_ranged_value as percentage_to_ranged_value, ranged_value_to_percentage as ranged_value_to_percentage
from typing import Any

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class BondFan(BondEntity, FanEntity):
    _power: Any
    _speed: Any
    _direction: Any
    def __init__(self, hub: BondHub, device: BondDevice, bpup_subs: BPUPSubscriptions) -> None: ...
    def _apply_state(self, state: dict) -> None: ...
    @property
    def supported_features(self) -> int: ...
    @property
    def _speed_range(self) -> tuple[int, int]: ...
    @property
    def percentage(self) -> int: ...
    @property
    def speed_count(self) -> int: ...
    @property
    def current_direction(self) -> Union[str, None]: ...
    async def async_set_percentage(self, percentage: int) -> None: ...
    async def async_turn_on(self, speed: Union[str, None] = ..., percentage: Union[int, None] = ..., preset_mode: Union[str, None] = ..., **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...
    async def async_set_direction(self, direction: str) -> None: ...



from . import NotionEntity as NotionEntity
from .const import DATA_COORDINATOR as DATA_COORDINATOR, DOMAIN as DOMAIN, LOGGER as LOGGER, SENSOR_TEMPERATURE as SENSOR_TEMPERATURE
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any

SENSOR_TYPES: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class NotionSensor(NotionEntity, SensorEntity):
    _attr_unit_of_measurement: Any
    def __init__(self, coordinator: DataUpdateCoordinator, task_id: str, sensor_id: str, bridge_id: str, system_id: str, name: str, device_class: str, unit: str) -> None: ...
    _attr_state: Any
    def _async_update_from_latest_data(self) -> None: ...



from . import EsphomeEntity as EsphomeEntity, platform_async_setup_entry as platform_async_setup_entry
from aioesphomeapi import BinarySensorInfo, BinarySensorState
from homeassistant.components.binary_sensor import BinarySensorEntity as BinarySensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class EsphomeBinarySensor(EsphomeEntity[BinarySensorInfo, BinarySensorState], BinarySensorEntity):
    @property
    def is_on(self) -> Union[bool, None]: ...
    @property
    def device_class(self) -> str: ...
    @property
    def available(self) -> bool: ...



from collections.abc import Awaitable
from homeassistant.core import Event as Event, HomeAssistant as HomeAssistant
from homeassistant.loader import async_get_integration as async_get_integration, bind_hass as bind_hass
from homeassistant.setup import ATTR_COMPONENT as ATTR_COMPONENT, EVENT_COMPONENT_LOADED as EVENT_COMPONENT_LOADED
from typing import Any, Callable

_LOGGER: Any

async def async_process_integration_platforms(hass: HomeAssistant, platform_name: str, process_platform: Callable[[HomeAssistant, str, Any], Awaitable[None]]) -> None: ...



from collections.abc import Iterable, Mapping
from contextvars import ContextVar
from enum import Enum
from homeassistant import data_entry_flow as data_entry_flow, loader as loader
from homeassistant.const import EVENT_HOMEASSISTANT_STARTED as EVENT_HOMEASSISTANT_STARTED, EVENT_HOMEASSISTANT_STOP as EVENT_HOMEASSISTANT_STOP
from homeassistant.core import CALLBACK_TYPE as CALLBACK_TYPE, CoreState as CoreState, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import ConfigEntryAuthFailed as ConfigEntryAuthFailed, ConfigEntryNotReady as ConfigEntryNotReady, HomeAssistantError as HomeAssistantError
from homeassistant.helpers import device_registry as device_registry, entity_registry as entity_registry
from homeassistant.helpers.event import Event as Event
from homeassistant.helpers.typing import DiscoveryInfoType as DiscoveryInfoType, UNDEFINED as UNDEFINED, UndefinedType as UndefinedType
from homeassistant.setup import async_process_deps_reqs as async_process_deps_reqs, async_setup_component as async_setup_component
from homeassistant.util.decorator import Registry as Registry
from typing import Any

_LOGGER: Any
SOURCE_DISCOVERY: str
SOURCE_HASSIO: str
SOURCE_HOMEKIT: str
SOURCE_IMPORT: str
SOURCE_INTEGRATION_DISCOVERY: str
SOURCE_MQTT: str
SOURCE_SSDP: str
SOURCE_USER: str
SOURCE_ZEROCONF: str
SOURCE_DHCP: str
SOURCE_IGNORE: str
SOURCE_UNIGNORE: str
SOURCE_REAUTH: str
HANDLERS: Any
STORAGE_KEY: str
STORAGE_VERSION: int
PATH_CONFIG: str
SAVE_DELAY: int

class ConfigEntryState(Enum):
    LOADED: Any
    SETUP_ERROR: Any
    MIGRATION_ERROR: Any
    SETUP_RETRY: Any
    NOT_LOADED: Any
    FAILED_UNLOAD: Any
    _recoverable: bool
    def __new__(cls, value: str, recoverable: bool) -> ConfigEntryState: ...
    @property
    def recoverable(self) -> bool: ...

DEFAULT_DISCOVERY_UNIQUE_ID: str
DISCOVERY_NOTIFICATION_ID: str
DISCOVERY_SOURCES: Any
RECONFIGURE_NOTIFICATION_ID: str
EVENT_FLOW_DISCOVERED: str
DISABLED_USER: str
RELOAD_AFTER_UPDATE_DELAY: int
CONN_CLASS_CLOUD_PUSH: str
CONN_CLASS_CLOUD_POLL: str
CONN_CLASS_LOCAL_PUSH: str
CONN_CLASS_LOCAL_POLL: str
CONN_CLASS_ASSUMED: str
CONN_CLASS_UNKNOWN: str

class ConfigError(HomeAssistantError): ...
class UnknownEntry(ConfigError): ...
class OperationNotAllowed(ConfigError): ...

UpdateListenerType: Any

class ConfigEntry:
    __slots__: Any
    entry_id: Any
    version: Any
    domain: Any
    title: Any
    data: Any
    options: Any
    pref_disable_new_entities: Any
    pref_disable_polling: Any
    source: Any
    state: Any
    unique_id: Any
    disabled_by: Any
    supports_unload: bool
    update_listeners: Any
    reason: Any
    _async_cancel_retry_setup: Any
    _on_unload: Any
    def __init__(self, version: int, domain: str, title: str, data: Mapping[str, Any], source: str, pref_disable_new_entities: Union[bool, None] = ..., pref_disable_polling: Union[bool, None] = ..., options: Union[Mapping[str, Any], None] = ..., unique_id: Union[str, None] = ..., entry_id: Union[str, None] = ..., state: ConfigEntryState = ..., disabled_by: Union[str, None] = ...) -> None: ...
    async def async_setup(self, hass: HomeAssistant, *, integration: Union[loader.Integration, None] = ..., tries: int = ...) -> None: ...
    async def async_shutdown(self) -> None: ...
    def async_cancel_retry_setup(self) -> None: ...
    async def async_unload(self, hass: HomeAssistant, *, integration: Union[loader.Integration, None] = ...) -> bool: ...
    async def async_remove(self, hass: HomeAssistant) -> None: ...
    async def async_migrate(self, hass: HomeAssistant) -> bool: ...
    def add_update_listener(self, listener: UpdateListenerType) -> CALLBACK_TYPE: ...
    def as_dict(self) -> dict[str, Any]: ...
    def async_on_unload(self, func: CALLBACK_TYPE) -> None: ...
    def _async_process_on_unload(self) -> None: ...
    def async_start_reauth(self, hass: HomeAssistant) -> None: ...

current_entry: ContextVar[Union[ConfigEntry, None]]

class ConfigEntriesFlowManager(data_entry_flow.FlowManager):
    config_entries: Any
    _hass_config: Any
    def __init__(self, hass: HomeAssistant, config_entries: ConfigEntries, hass_config: dict) -> None: ...
    async def async_finish_flow(self, flow: data_entry_flow.FlowHandler, result: data_entry_flow.FlowResult) -> data_entry_flow.FlowResult: ...
    async def async_create_flow(self, handler_key: Any, *, context: Union[dict, None] = ..., data: Any = ...) -> ConfigFlow: ...
    async def async_post_init(self, flow: data_entry_flow.FlowHandler, result: data_entry_flow.FlowResult) -> None: ...

class ConfigEntries:
    hass: Any
    flow: Any
    options: Any
    _hass_config: Any
    _entries: Any
    _store: Any
    def __init__(self, hass: HomeAssistant, hass_config: dict) -> None: ...
    def async_domains(self, include_ignore: bool = ..., include_disabled: bool = ...) -> list[str]: ...
    def async_get_entry(self, entry_id: str) -> Union[ConfigEntry, None]: ...
    def async_entries(self, domain: Union[str, None] = ...) -> list[ConfigEntry]: ...
    async def async_add(self, entry: ConfigEntry) -> None: ...
    async def async_remove(self, entry_id: str) -> dict[str, Any]: ...
    async def _async_shutdown(self, event: Event) -> None: ...
    async def async_initialize(self) -> None: ...
    async def async_setup(self, entry_id: str) -> bool: ...
    async def async_unload(self, entry_id: str) -> bool: ...
    async def async_reload(self, entry_id: str) -> bool: ...
    async def async_set_disabled_by(self, entry_id: str, disabled_by: Union[str, None]) -> bool: ...
    def async_update_entry(self, entry: ConfigEntry, *, unique_id: Union[str, None, UndefinedType] = ..., title: Union[str, UndefinedType] = ..., data: Union[dict, UndefinedType] = ..., options: Union[Mapping[str, Any], UndefinedType] = ..., pref_disable_new_entities: Union[bool, UndefinedType] = ..., pref_disable_polling: Union[bool, UndefinedType] = ...) -> bool: ...
    def async_setup_platforms(self, entry: ConfigEntry, platforms: Iterable[str]) -> None: ...
    async def async_forward_entry_setup(self, entry: ConfigEntry, domain: str) -> bool: ...
    async def async_unload_platforms(self, entry: ConfigEntry, platforms: Iterable[str]) -> bool: ...
    async def async_forward_entry_unload(self, entry: ConfigEntry, domain: str) -> bool: ...
    def _async_schedule_save(self) -> None: ...
    def _data_to_save(self) -> dict[str, list[dict[str, Any]]]: ...

async def _old_conf_migrator(old_config: dict[str, Any]) -> dict[str, Any]: ...

class ConfigFlow(data_entry_flow.FlowHandler):
    def __init_subclass__(cls, domain: Union[str, None] = ..., **kwargs: Any) -> None: ...
    @property
    def unique_id(self) -> Union[str, None]: ...
    @staticmethod
    def async_get_options_flow(config_entry: ConfigEntry) -> OptionsFlow: ...
    def _async_abort_entries_match(self, match_dict: Union[dict[str, Any], None] = ...) -> None: ...
    def _abort_if_unique_id_configured(self, updates: Union[dict[Any, Any], None] = ..., reload_on_update: bool = ...) -> None: ...
    async def async_set_unique_id(self, unique_id: Union[str, None] = ..., *, raise_on_progress: bool = ...) -> Union[ConfigEntry, None]: ...
    def _set_confirm_only(self) -> None: ...
    def _async_current_entries(self, include_ignore: Union[bool, None] = ...) -> list[ConfigEntry]: ...
    def _async_current_ids(self, include_ignore: bool = ...) -> set[Union[str, None]]: ...
    def _async_in_progress(self, include_uninitialized: bool = ...) -> list[data_entry_flow.FlowResult]: ...
    async def async_step_ignore(self, user_input: dict[str, Any]) -> data_entry_flow.FlowResult: ...
    async def async_step_unignore(self, user_input: dict[str, Any]) -> data_entry_flow.FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> data_entry_flow.FlowResult: ...
    async def _async_handle_discovery_without_unique_id(self) -> None: ...
    async def async_step_discovery(self, discovery_info: DiscoveryInfoType) -> data_entry_flow.FlowResult: ...
    def async_abort(self, reason: str, *, description_placeholders: Union[dict, None] = ...) -> data_entry_flow.FlowResult: ...
    async def async_step_hassio(self, discovery_info: DiscoveryInfoType) -> data_entry_flow.FlowResult: ...
    async def async_step_homekit(self, discovery_info: DiscoveryInfoType) -> data_entry_flow.FlowResult: ...
    async def async_step_mqtt(self, discovery_info: DiscoveryInfoType) -> data_entry_flow.FlowResult: ...
    async def async_step_ssdp(self, discovery_info: DiscoveryInfoType) -> data_entry_flow.FlowResult: ...
    async def async_step_zeroconf(self, discovery_info: DiscoveryInfoType) -> data_entry_flow.FlowResult: ...
    async def async_step_dhcp(self, discovery_info: DiscoveryInfoType) -> data_entry_flow.FlowResult: ...
    def async_create_entry(self, title: str, data: Mapping[str, Any], *, description: Union[str, None] = ..., description_placeholders: Union[dict, None] = ..., options: Union[Mapping[str, Any], None] = ...) -> data_entry_flow.FlowResult: ...

class OptionsFlowManager(data_entry_flow.FlowManager):
    async def async_create_flow(self, handler_key: Any, *, context: Union[dict[str, Any], None] = ..., data: Union[dict[str, Any], None] = ...) -> OptionsFlow: ...
    async def async_finish_flow(self, flow: data_entry_flow.FlowHandler, result: data_entry_flow.FlowResult) -> data_entry_flow.FlowResult: ...

class OptionsFlow(data_entry_flow.FlowHandler):
    handler: str

class EntityRegistryDisabledHandler:
    hass: Any
    registry: Any
    changed: Any
    _remove_call_later: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    def async_setup(self) -> None: ...
    async def _handle_entry_updated(self, event: Event) -> None: ...
    async def _handle_reload(self, _now: Any) -> None: ...

def _handle_entry_updated_filter(event: Event) -> bool: ...
async def support_entry_unload(hass: HomeAssistant, domain: str) -> bool: ...



import voluptuous as vol
from . import async_get_client_id as async_get_client_id
from .const import DOMAIN as DOMAIN, LOGGER as LOGGER
from homeassistant import config_entries as config_entries
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_CODE as CONF_CODE, CONF_PASSWORD as CONF_PASSWORD, CONF_USERNAME as CONF_USERNAME
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers import aiohttp_client as aiohttp_client
from homeassistant.helpers.typing import ConfigType as ConfigType
from simplipy.api import API as API
from typing import Any

FULL_DATA_SCHEMA: Any
PASSWORD_DATA_SCHEMA: Any

class SimpliSafeFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    _code: Any
    _password: Any
    _username: Any
    def __init__(self) -> None: ...
    @staticmethod
    def async_get_options_flow(config_entry: ConfigEntry) -> SimpliSafeOptionsFlowHandler: ...
    async def _async_get_simplisafe_api(self) -> API: ...
    async def _async_login_during_step(self, step_id: str, form_schema: vol.Schema) -> FlowResult: ...
    async def async_step_finish(self, user_input: dict[str, Any]) -> FlowResult: ...
    async def async_step_mfa(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_reauth(self, config: ConfigType) -> FlowResult: ...
    async def async_step_reauth_confirm(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...

class SimpliSafeOptionsFlowHandler(config_entries.OptionsFlow):
    config_entry: Any
    def __init__(self, config_entry: ConfigEntry) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...



from homeassistant.const import PRECISION_HALVES as PRECISION_HALVES, PRECISION_TENTHS as PRECISION_TENTHS
from homeassistant.core import HomeAssistant as HomeAssistant

def display_temp(hass: HomeAssistant, temperature: Union[float, None], unit: str, precision: float) -> Union[float, None]: ...



from .const import ATTR_INPUT_SOURCE as ATTR_INPUT_SOURCE, ATTR_MEDIA_CONTENT_ID as ATTR_MEDIA_CONTENT_ID, ATTR_MEDIA_CONTENT_TYPE as ATTR_MEDIA_CONTENT_TYPE, ATTR_MEDIA_ENQUEUE as ATTR_MEDIA_ENQUEUE, ATTR_MEDIA_VOLUME_LEVEL as ATTR_MEDIA_VOLUME_LEVEL, ATTR_MEDIA_VOLUME_MUTED as ATTR_MEDIA_VOLUME_MUTED, ATTR_SOUND_MODE as ATTR_SOUND_MODE, DOMAIN as DOMAIN, SERVICE_PLAY_MEDIA as SERVICE_PLAY_MEDIA, SERVICE_SELECT_SOUND_MODE as SERVICE_SELECT_SOUND_MODE, SERVICE_SELECT_SOURCE as SERVICE_SELECT_SOURCE
from collections.abc import Iterable
from homeassistant.const import SERVICE_MEDIA_PAUSE as SERVICE_MEDIA_PAUSE, SERVICE_MEDIA_PLAY as SERVICE_MEDIA_PLAY, SERVICE_MEDIA_STOP as SERVICE_MEDIA_STOP, SERVICE_TURN_OFF as SERVICE_TURN_OFF, SERVICE_TURN_ON as SERVICE_TURN_ON, SERVICE_VOLUME_MUTE as SERVICE_VOLUME_MUTE, SERVICE_VOLUME_SET as SERVICE_VOLUME_SET, STATE_IDLE as STATE_IDLE, STATE_OFF as STATE_OFF, STATE_ON as STATE_ON, STATE_PAUSED as STATE_PAUSED, STATE_PLAYING as STATE_PLAYING
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant, State as State
from typing import Any

async def _async_reproduce_states(hass: HomeAssistant, state: State, *, context: Union[Context, None] = ..., reproduce_options: Union[dict[str, Any], None] = ...) -> None: ...
async def async_reproduce_states(hass: HomeAssistant, states: Iterable[State], *, context: Union[Context, None] = ..., reproduce_options: Union[dict[str, Any], None] = ...) -> None: ...



import asyncio
from homeassistant import bootstrap as bootstrap
from homeassistant.core import callback as callback
from homeassistant.helpers.frame import warn_use as warn_use
from homeassistant.util.executor import InterruptibleThreadPoolExecutor as InterruptibleThreadPoolExecutor
from homeassistant.util.thread import deadlock_safe_shutdown as deadlock_safe_shutdown
from typing import Any

MAX_EXECUTOR_WORKERS: int

class RuntimeConfig:
    config_dir: str
    skip_pip: bool
    safe_mode: bool
    verbose: bool
    log_rotate_days: Union[int, None]
    log_file: Union[str, None]
    log_no_color: bool
    debug: bool
    open_ui: bool

class HassEventLoopPolicy(asyncio.DefaultEventLoopPolicy):
    debug: Any
    def __init__(self, debug: bool) -> None: ...
    @property
    def loop_name(self) -> str: ...
    def new_event_loop(self) -> asyncio.AbstractEventLoop: ...

def _async_loop_exception_handler(_: Any, context: dict[str, Any]) -> None: ...
async def setup_and_run_hass(runtime_config: RuntimeConfig) -> int: ...
def run(runtime_config: RuntimeConfig) -> int: ...



from .const import CONF_RESPOND_TO_READ as CONF_RESPOND_TO_READ, DOMAIN as DOMAIN, KNX_ADDRESS as KNX_ADDRESS
from .knx_entity import KnxEntity as KnxEntity
from .schema import SwitchSchema as SwitchSchema
from homeassistant.components.switch import SwitchEntity as SwitchEntity
from homeassistant.const import CONF_NAME as CONF_NAME, STATE_ON as STATE_ON, STATE_UNAVAILABLE as STATE_UNAVAILABLE, STATE_UNKNOWN as STATE_UNKNOWN
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.restore_state import RestoreEntity as RestoreEntity
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any
from xknx import XKNX as XKNX
from xknx.devices import Switch as XknxSwitch

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class KNXSwitch(KnxEntity, SwitchEntity, RestoreEntity):
    _device: XknxSwitch
    _attr_unique_id: Any
    def __init__(self, xknx: XKNX, config: ConfigType) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    @property
    def is_on(self) -> bool: ...
    async def async_turn_on(self, **kwargs: Any) -> None: ...
    async def async_turn_off(self, **kwargs: Any) -> None: ...



from .const import DOMAIN as DOMAIN
from .devolo_multi_level_switch import DevoloMultiLevelSwitchDeviceEntity as DevoloMultiLevelSwitchDeviceEntity
from devolo_home_control_api.devices.zwave import Zwave as Zwave
from devolo_home_control_api.homecontrol import HomeControl as HomeControl
from homeassistant.components.climate import ATTR_TEMPERATURE as ATTR_TEMPERATURE, ClimateEntity as ClimateEntity, HVAC_MODE_HEAT as HVAC_MODE_HEAT, SUPPORT_TARGET_TEMPERATURE as SUPPORT_TARGET_TEMPERATURE, TEMP_CELSIUS as TEMP_CELSIUS
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import PRECISION_HALVES as PRECISION_HALVES, PRECISION_TENTHS as PRECISION_TENTHS
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class DevoloClimateDeviceEntity(DevoloMultiLevelSwitchDeviceEntity, ClimateEntity):
    _attr_hvac_mode: Any
    _attr_hvac_modes: Any
    _attr_min_temp: Any
    _attr_max_temp: Any
    _attr_precision: Any
    _attr_supported_features: Any
    _attr_target_temperature_step: Any
    _attr_temperature_unit: Any
    def __init__(self, homecontrol: HomeControl, device_instance: Zwave, element_uid: str) -> None: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature(self) -> Union[float, None]: ...
    def set_hvac_mode(self, hvac_mode: str) -> None: ...
    def set_temperature(self, **kwargs: Any) -> None: ...



from . import BraviaTVCoordinator as BraviaTVCoordinator
from .const import ATTR_MANUFACTURER as ATTR_MANUFACTURER, DEFAULT_NAME as DEFAULT_NAME, DOMAIN as DOMAIN
from homeassistant.components.media_player import DEVICE_CLASS_TV as DEVICE_CLASS_TV, MediaPlayerEntity as MediaPlayerEntity
from homeassistant.components.media_player.const import SUPPORT_NEXT_TRACK as SUPPORT_NEXT_TRACK, SUPPORT_PAUSE as SUPPORT_PAUSE, SUPPORT_PLAY as SUPPORT_PLAY, SUPPORT_PREVIOUS_TRACK as SUPPORT_PREVIOUS_TRACK, SUPPORT_SELECT_SOURCE as SUPPORT_SELECT_SOURCE, SUPPORT_STOP as SUPPORT_STOP, SUPPORT_TURN_OFF as SUPPORT_TURN_OFF, SUPPORT_TURN_ON as SUPPORT_TURN_ON, SUPPORT_VOLUME_MUTE as SUPPORT_VOLUME_MUTE, SUPPORT_VOLUME_SET as SUPPORT_VOLUME_SET, SUPPORT_VOLUME_STEP as SUPPORT_VOLUME_STEP
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import STATE_OFF as STATE_OFF, STATE_PAUSED as STATE_PAUSED, STATE_PLAYING as STATE_PLAYING
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any, Final

SUPPORT_BRAVIA: Final[Any]

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class BraviaTVMediaPlayer(CoordinatorEntity, MediaPlayerEntity):
    coordinator: BraviaTVCoordinator
    _attr_device_class: Any
    _attr_supported_features: Any
    _attr_device_info: Any
    _attr_name: Any
    _attr_unique_id: Any
    def __init__(self, coordinator: BraviaTVCoordinator, name: str, unique_id: str, device_info: DeviceInfo) -> None: ...
    @property
    def state(self) -> Union[str, None]: ...
    @property
    def source(self) -> Union[str, None]: ...
    @property
    def source_list(self) -> list[str]: ...
    @property
    def volume_level(self) -> Union[float, None]: ...
    @property
    def is_volume_muted(self) -> bool: ...
    @property
    def media_title(self) -> Union[str, None]: ...
    @property
    def media_content_id(self) -> Union[str, None]: ...
    @property
    def media_duration(self) -> Union[int, None]: ...
    async def async_turn_on(self) -> None: ...
    async def async_turn_off(self) -> None: ...
    async def async_set_volume_level(self, volume: float) -> None: ...
    async def async_volume_up(self) -> None: ...
    async def async_volume_down(self) -> None: ...
    async def async_mute_volume(self, mute: bool) -> None: ...
    async def async_select_source(self, source: str) -> None: ...
    async def async_media_play(self) -> None: ...
    async def async_media_pause(self) -> None: ...
    async def async_media_stop(self) -> None: ...
    async def async_media_next_track(self) -> None: ...
    async def async_media_previous_track(self) -> None: ...



from .const import DEVICE_ID as DEVICE_ID, DOMAIN as DOMAIN, EVENT_TAG_SCANNED as EVENT_TAG_SCANNED, TAG_ID as TAG_ID
from homeassistant.const import CONF_NAME as CONF_NAME
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from homeassistant.helpers import collection as collection
from homeassistant.helpers.storage import Store as Store
from homeassistant.helpers.typing import ConfigType as ConfigType
from homeassistant.loader import bind_hass as bind_hass
from typing import Any

_LOGGER: Any
LAST_SCANNED: str
STORAGE_KEY = DOMAIN
STORAGE_VERSION: int
TAGS: str
CREATE_FIELDS: Any
UPDATE_FIELDS: Any

class TagIDExistsError(HomeAssistantError):
    item_id: Any
    def __init__(self, item_id: str) -> None: ...

class TagIDManager(collection.IDManager):
    def generate_id(self, suggestion: str) -> str: ...

class TagStorageCollection(collection.StorageCollection):
    CREATE_SCHEMA: Any
    UPDATE_SCHEMA: Any
    async def _process_create_data(self, data: dict) -> dict: ...
    def _get_suggested_id(self, info: dict[str, str]) -> str: ...
    async def _update_data(self, data: dict, update_data: dict) -> dict: ...

async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool: ...
async def async_scan_tag(hass: HomeAssistant, tag_id: str, device_id: str, context: Union[Context, None] = ...) -> None: ...



from collections.abc import Hashable
from typing import Callable, TypeVar

CALLABLE_T = TypeVar('CALLABLE_T', bound=Callable)

class Registry(dict):
    def register(self, name: Hashable) -> Callable[[CALLABLE_T], CALLABLE_T]: ...



from . import EsphomeEntity as EsphomeEntity, esphome_state_property as esphome_state_property, platform_async_setup_entry as platform_async_setup_entry
from aioesphomeapi import NumberInfo, NumberState
from homeassistant.components.number import NumberEntity as NumberEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

ICON_SCHEMA: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class EsphomeNumber(EsphomeEntity[NumberInfo, NumberState], NumberEntity):
    @property
    def icon(self) -> Union[str, None]: ...
    @property
    def min_value(self) -> float: ...
    @property
    def max_value(self) -> float: ...
    @property
    def step(self) -> float: ...
    def value(self) -> Union[float, None]: ...
    async def async_set_value(self, value: float) -> None: ...



import aiohttp
from typing import Any, NamedTuple

WHOAMI_URL: str
WHOAMI_URL_DEV: str
AXIS_A: int
FLATTENING: Any
AXIS_B: float
MILES_PER_KILOMETER: float
MAX_ITERATIONS: int
CONVERGENCE_THRESHOLD: float

class LocationInfo(NamedTuple):
    ip: Any
    country_code: Any
    currency: Any
    region_code: Any
    region_name: Any
    city: Any
    zip_code: Any
    time_zone: Any
    latitude: Any
    longitude: Any
    use_metric: Any

async def async_detect_location_info(session: aiohttp.ClientSession) -> Union[LocationInfo, None]: ...
def distance(lat1: Union[float, None], lon1: Union[float, None], lat2: float, lon2: float) -> Union[float, None]: ...
def vincenty(point1: tuple[float, float], point2: tuple[float, float], miles: bool = ...) -> Union[float, None]: ...
async def _get_whoami(session: aiohttp.ClientSession) -> Union[dict[str, Any], None]: ...



from .const import DevId as DevId, DiscoveryInfo as DiscoveryInfo
from homeassistant.components import mysensors as mysensors
from homeassistant.components.notify import ATTR_TARGET as ATTR_TARGET, BaseNotificationService as BaseNotificationService, DOMAIN as DOMAIN
from homeassistant.core import HomeAssistant as HomeAssistant
from typing import Any

async def async_get_service(hass: HomeAssistant, config: dict[str, Any], discovery_info: Union[DiscoveryInfo, None] = ...) -> Union[BaseNotificationService, None]: ...

class MySensorsNotificationDevice(mysensors.device.MySensorsDevice):
    def send_msg(self, msg: str) -> None: ...
    def __repr__(self) -> str: ...

class MySensorsNotificationService(BaseNotificationService):
    devices: Any
    def __init__(self, hass: HomeAssistant) -> None: ...
    async def async_send_message(self, message: str = ..., **kwargs: Any) -> None: ...



from .const import CONF_DSMR_VERSION as CONF_DSMR_VERSION, CONF_SERIAL_ID as CONF_SERIAL_ID, CONF_SERIAL_ID_GAS as CONF_SERIAL_ID_GAS, CONF_TIME_BETWEEN_UPDATE as CONF_TIME_BETWEEN_UPDATE, DEFAULT_TIME_BETWEEN_UPDATE as DEFAULT_TIME_BETWEEN_UPDATE, DOMAIN as DOMAIN, LOGGER as LOGGER
from dsmr_parser.objects import DSMRObject as DSMRObject
from homeassistant import config_entries as config_entries, core as core, exceptions as exceptions
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import CONF_HOST as CONF_HOST, CONF_PORT as CONF_PORT, CONF_TYPE as CONF_TYPE
from homeassistant.core import callback as callback
from homeassistant.data_entry_flow import FlowResult as FlowResult
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

CONF_MANUAL_PATH: str

class DSMRConnection:
    _host: Any
    _port: Any
    _dsmr_version: Any
    _telegram: Any
    _equipment_identifier: Any
    def __init__(self, host: Union[str, None], port: int, dsmr_version: str) -> None: ...
    def equipment_identifier(self) -> Union[str, None]: ...
    def equipment_identifier_gas(self) -> Union[str, None]: ...
    async def validate_connect(self, hass: core.HomeAssistant) -> bool: ...

async def _validate_dsmr_connection(hass: core.HomeAssistant, data: dict[str, Any]) -> dict[str, Union[str, None]]: ...

class DSMRFlowHandler(config_entries.ConfigFlow):
    VERSION: int
    _dsmr_version: Union[str, None]
    @staticmethod
    def async_get_options_flow(config_entry: ConfigEntry) -> DSMROptionFlowHandler: ...
    def _abort_if_host_port_configured(self, port: str, host: Union[str, None] = ..., updates: Union[dict[Any, Any], None] = ..., reload_on_update: bool = ...) -> Union[FlowResult, None]: ...
    async def async_step_user(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_setup_network(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_setup_serial(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_step_setup_serial_manual_path(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...
    async def async_validate_dsmr(self, input_data: dict[str, Any], errors: dict[str, str]) -> dict[str, Any]: ...
    async def async_step_import(self, import_config: ConfigType) -> FlowResult: ...

class DSMROptionFlowHandler(config_entries.OptionsFlow):
    entry: Any
    def __init__(self, entry: ConfigEntry) -> None: ...
    async def async_step_init(self, user_input: Union[dict[str, Any], None] = ...) -> FlowResult: ...

def get_serial_by_id(dev_path: str) -> str: ...

class CannotConnect(exceptions.HomeAssistantError): ...
class CannotCommunicate(exceptions.HomeAssistantError): ...



from homeassistant.const import CONF_MODE as CONF_MODE, CONF_UNIT_OF_MEASUREMENT as CONF_UNIT_OF_MEASUREMENT
from homeassistant.util import decorator as decorator
from typing import Any, Callable

SELECTORS: Any

def validate_selector(config: Any) -> dict: ...

class Selector:
    CONFIG_SCHEMA: Callable

class EntitySelector(Selector):
    CONFIG_SCHEMA: Any

class DeviceSelector(Selector):
    CONFIG_SCHEMA: Any

class AreaSelector(Selector):
    CONFIG_SCHEMA: Any

class NumberSelector(Selector):
    CONFIG_SCHEMA: Any

class AddonSelector(Selector):
    CONFIG_SCHEMA: Any

class BooleanSelector(Selector):
    CONFIG_SCHEMA: Any

class TimeSelector(Selector):
    CONFIG_SCHEMA: Any

class TargetSelector(Selector):
    CONFIG_SCHEMA: Any

class ActionSelector(Selector):
    CONFIG_SCHEMA: Any

class ObjectSelector(Selector):
    CONFIG_SCHEMA: Any

class StringSelector(Selector):
    CONFIG_SCHEMA: Any

class SelectSelector(Selector):
    CONFIG_SCHEMA: Any



from . import DOMAIN as DOMAIN
from collections.abc import Iterable
from homeassistant.components.cover import ATTR_CURRENT_POSITION as ATTR_CURRENT_POSITION, ATTR_CURRENT_TILT_POSITION as ATTR_CURRENT_TILT_POSITION, ATTR_POSITION as ATTR_POSITION, ATTR_TILT_POSITION as ATTR_TILT_POSITION
from homeassistant.const import ATTR_ENTITY_ID as ATTR_ENTITY_ID, SERVICE_CLOSE_COVER as SERVICE_CLOSE_COVER, SERVICE_CLOSE_COVER_TILT as SERVICE_CLOSE_COVER_TILT, SERVICE_OPEN_COVER as SERVICE_OPEN_COVER, SERVICE_OPEN_COVER_TILT as SERVICE_OPEN_COVER_TILT, SERVICE_SET_COVER_POSITION as SERVICE_SET_COVER_POSITION, SERVICE_SET_COVER_TILT_POSITION as SERVICE_SET_COVER_TILT_POSITION, STATE_CLOSED as STATE_CLOSED, STATE_CLOSING as STATE_CLOSING, STATE_OPEN as STATE_OPEN, STATE_OPENING as STATE_OPENING
from homeassistant.core import Context as Context, HomeAssistant as HomeAssistant, State as State
from typing import Any

_LOGGER: Any
VALID_STATES: Any

async def _async_reproduce_state(hass: HomeAssistant, state: State, *, context: Union[Context, None] = ..., reproduce_options: Union[dict[str, Any], None] = ...) -> None: ...
async def async_reproduce_states(hass: HomeAssistant, states: Iterable[State], *, context: Union[Context, None] = ..., reproduce_options: Union[dict[str, Any], None] = ...) -> None: ...



from .helpers import on_unload as on_unload
from homeassistant.components import mysensors as mysensors
from homeassistant.components.climate import ClimateEntity as ClimateEntity
from homeassistant.components.climate.const import ATTR_TARGET_TEMP_HIGH as ATTR_TARGET_TEMP_HIGH, ATTR_TARGET_TEMP_LOW as ATTR_TARGET_TEMP_LOW, DOMAIN as DOMAIN, HVAC_MODE_AUTO as HVAC_MODE_AUTO, HVAC_MODE_COOL as HVAC_MODE_COOL, HVAC_MODE_HEAT as HVAC_MODE_HEAT, HVAC_MODE_OFF as HVAC_MODE_OFF, SUPPORT_FAN_MODE as SUPPORT_FAN_MODE, SUPPORT_TARGET_TEMPERATURE as SUPPORT_TARGET_TEMPERATURE, SUPPORT_TARGET_TEMPERATURE_RANGE as SUPPORT_TARGET_TEMPERATURE_RANGE
from homeassistant.components.mysensors.const import DiscoveryInfo as DiscoveryInfo, MYSENSORS_DISCOVERY as MYSENSORS_DISCOVERY
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import ATTR_TEMPERATURE as ATTR_TEMPERATURE, TEMP_CELSIUS as TEMP_CELSIUS, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

DICT_HA_TO_MYS: Any
DICT_MYS_TO_HA: Any
FAN_LIST: Any
OPERATION_LIST: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class MySensorsHVAC(mysensors.device.MySensorsEntity, ClimateEntity):
    @property
    def supported_features(self) -> int: ...
    @property
    def temperature_unit(self) -> str: ...
    @property
    def current_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature(self) -> Union[float, None]: ...
    @property
    def target_temperature_high(self) -> Union[float, None]: ...
    @property
    def target_temperature_low(self) -> Union[float, None]: ...
    @property
    def hvac_mode(self) -> str: ...
    @property
    def hvac_modes(self) -> list[str]: ...
    @property
    def fan_mode(self) -> Union[str, None]: ...
    @property
    def fan_modes(self) -> list[str]: ...
    async def async_set_temperature(self, **kwargs: Any) -> None: ...
    async def async_set_fan_mode(self, fan_mode: str) -> None: ...
    async def async_set_hvac_mode(self, hvac_mode: str) -> None: ...
    async def async_update(self) -> None: ...



import concurrent.futures
from asyncio.events import AbstractEventLoop
from collections.abc import Coroutine
from typing import Any, Callable, TypeVar

_LOGGER: Any
_SHUTDOWN_RUN_CALLBACK_THREADSAFE: str
T = TypeVar('T')

def fire_coroutine_threadsafe(coro: Coroutine, loop: AbstractEventLoop) -> None: ...
def run_callback_threadsafe(loop: AbstractEventLoop, callback: Callable[..., T], *args: Any) -> concurrent.futures.Future[T]: ...
def check_loop() -> None: ...
def protect_loop(func: Callable) -> Callable: ...
async def gather_with_concurrency(limit: int, *tasks: Any, return_exceptions: bool = ...) -> Any: ...
def shutdown_run_callback_threadsafe(loop: AbstractEventLoop) -> None: ...



from . import KrakenData as KrakenData
from .const import CONF_TRACKED_ASSET_PAIRS as CONF_TRACKED_ASSET_PAIRS, DISPATCH_CONFIG_UPDATED as DISPATCH_CONFIG_UPDATED, DOMAIN as DOMAIN, SENSOR_TYPES as SENSOR_TYPES, SensorType as SensorType
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers import device_registry as device_registry
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity import DeviceInfo as DeviceInfo
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import StateType as StateType
from homeassistant.helpers.update_coordinator import CoordinatorEntity as CoordinatorEntity
from typing import Any

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class KrakenSensor(CoordinatorEntity, SensorEntity):
    tracked_asset_pair_wsname: Any
    _source_asset: Any
    _target_asset: Any
    _sensor_type: Any
    _enabled_by_default: Any
    _unit_of_measurement: Any
    _device_name: Any
    _name: Any
    _received_data_at_least_once: bool
    _available: bool
    _state: Any
    def __init__(self, kraken_data: KrakenData, tracked_asset_pair: str, sensor_type: SensorType) -> None: ...
    @property
    def entity_registry_enabled_default(self) -> bool: ...
    @property
    def name(self) -> str: ...
    @property
    def unique_id(self) -> str: ...
    @property
    def state(self) -> StateType: ...
    async def async_added_to_hass(self) -> None: ...
    def _handle_coordinator_update(self) -> None: ...
    def _update_internal_state(self) -> None: ...
    @property
    def icon(self) -> str: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...
    @property
    def available(self) -> bool: ...
    @property
    def device_info(self) -> DeviceInfo: ...

def create_device_name(tracked_asset_pair: str) -> str: ...



from homeassistant.exceptions import HomeAssistantError as HomeAssistantError
from traceback import FrameSummary
from typing import Any, Callable, TypeVar

_LOGGER: Any
CALLABLE_T = TypeVar('CALLABLE_T', bound=Callable)

def get_integration_frame(exclude_integrations: Union[set, None] = ...) -> tuple[FrameSummary, str, str]: ...

class MissingIntegrationFrame(HomeAssistantError): ...

def report(what: str) -> None: ...
def report_integration(what: str, integration_frame: tuple[FrameSummary, str, str]) -> None: ...
def warn_use(func: CALLABLE_T, what: str) -> CALLABLE_T: ...



import pyatmo
from .climate import get_all_home_ids as get_all_home_ids
from .const import DATA_HANDLER as DATA_HANDLER, DATA_SCHEDULES as DATA_SCHEDULES, DOMAIN as DOMAIN, EVENT_TYPE_SCHEDULE as EVENT_TYPE_SCHEDULE, MANUFACTURER as MANUFACTURER, SIGNAL_NAME as SIGNAL_NAME
from .data_handler import HOMEDATA_DATA_CLASS_NAME as HOMEDATA_DATA_CLASS_NAME, NetatmoDataHandler as NetatmoDataHandler
from .netatmo_entity_base import NetatmoBase as NetatmoBase
from homeassistant.components.select import SelectEntity as SelectEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.exceptions import PlatformNotReady as PlatformNotReady
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from typing import Any

_LOGGER: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class NetatmoScheduleSelect(NetatmoBase, SelectEntity):
    _home_id: Any
    _device_name: Any
    _attr_name: Any
    _model: str
    _attr_unique_id: Any
    _attr_current_option: Any
    _attr_options: Any
    def __init__(self, data_handler: NetatmoDataHandler, home_id: str, options: list) -> None: ...
    async def async_added_to_hass(self) -> None: ...
    async def handle_event(self, event: dict) -> None: ...
    @property
    def _data(self) -> pyatmo.AsyncHomeData: ...
    async def async_select_option(self, option: str) -> None: ...
    def async_update_callback(self) -> None: ...



from homeassistant.components.sensor import STATE_CLASS_MEASUREMENT as STATE_CLASS_MEASUREMENT, SensorEntityDescription as SensorEntityDescription
from homeassistant.const import CONCENTRATION_MICROGRAMS_PER_CUBIC_METER as CONCENTRATION_MICROGRAMS_PER_CUBIC_METER, CONCENTRATION_PARTS_PER_MILLION as CONCENTRATION_PARTS_PER_MILLION, DEVICE_CLASS_CO2 as DEVICE_CLASS_CO2, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_PRESSURE as DEVICE_CLASS_PRESSURE, DEVICE_CLASS_SIGNAL_STRENGTH as DEVICE_CLASS_SIGNAL_STRENGTH, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, DEVICE_CLASS_TIMESTAMP as DEVICE_CLASS_TIMESTAMP, PERCENTAGE as PERCENTAGE, PRESSURE_HPA as PRESSURE_HPA, SIGNAL_STRENGTH_DECIBELS_MILLIWATT as SIGNAL_STRENGTH_DECIBELS_MILLIWATT, TEMP_CELSIUS as TEMP_CELSIUS
from typing import Any, Final

SUFFIX_P0: Final[str]
SUFFIX_P1: Final[str]
SUFFIX_P2: Final[str]
SUFFIX_P4: Final[str]
ATTR_BME280_HUMIDITY: Final[str]
ATTR_BME280_PRESSURE: Final[str]
ATTR_BME280_TEMPERATURE: Final[str]
ATTR_BMP280_PRESSURE: Final[str]
ATTR_BMP280_TEMPERATURE: Final[str]
ATTR_DHT22_HUMIDITY: Final[str]
ATTR_DHT22_TEMPERATURE: Final[str]
ATTR_HECA_HUMIDITY: Final[str]
ATTR_HECA_TEMPERATURE: Final[str]
ATTR_MHZ14A_CARBON_DIOXIDE: Final[str]
ATTR_SDS011: Final[str]
ATTR_SDS011_P1: Final[Any]
ATTR_SDS011_P2: Final[Any]
ATTR_SHT3X_HUMIDITY: Final[str]
ATTR_SHT3X_TEMPERATURE: Final[str]
ATTR_SIGNAL_STRENGTH: Final[str]
ATTR_SPS30: Final[str]
ATTR_SPS30_P0: Final[Any]
ATTR_SPS30_P1: Final[Any]
ATTR_SPS30_P2: Final[Any]
ATTR_SPS30_P4: Final[Any]
ATTR_UPTIME: Final[str]
DEFAULT_NAME: Final[str]
DEFAULT_UPDATE_INTERVAL: Final[Any]
DOMAIN: Final[str]
MANUFACTURER: Final[str]
MIGRATION_SENSORS: Final[Any]
SENSORS: Final[tuple[SensorEntityDescription, ...]]



from .common import TCP_PLATFORM_SCHEMA as TCP_PLATFORM_SCHEMA, TcpEntity as TcpEntity
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.const import CONF_UNIT_OF_MEASUREMENT as CONF_UNIT_OF_MEASUREMENT
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, StateType as StateType
from typing import Any, Final

PLATFORM_SCHEMA: Final[Any]

def setup_platform(hass: HomeAssistant, config: ConfigType, add_entities: AddEntitiesCallback, discovery_info: Union[dict[str, Any], None] = ...) -> None: ...

class TcpSensor(TcpEntity, SensorEntity):
    @property
    def state(self) -> StateType: ...
    @property
    def unit_of_measurement(self) -> Union[str, None]: ...



from . import PairedSensorEntity as PairedSensorEntity, ValveControllerEntity as ValveControllerEntity
from .const import API_SYSTEM_DIAGNOSTICS as API_SYSTEM_DIAGNOSTICS, API_SYSTEM_ONBOARD_SENSOR_STATUS as API_SYSTEM_ONBOARD_SENSOR_STATUS, CONF_UID as CONF_UID, DATA_COORDINATOR as DATA_COORDINATOR, DATA_COORDINATOR_PAIRED_SENSOR as DATA_COORDINATOR_PAIRED_SENSOR, DATA_UNSUB_DISPATCHER_CONNECT as DATA_UNSUB_DISPATCHER_CONNECT, DOMAIN as DOMAIN, SIGNAL_PAIRED_SENSOR_COORDINATOR_ADDED as SIGNAL_PAIRED_SENSOR_COORDINATOR_ADDED
from homeassistant.components.sensor import SensorEntity as SensorEntity
from homeassistant.config_entries import ConfigEntry as ConfigEntry
from homeassistant.const import DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, PERCENTAGE as PERCENTAGE, TEMP_FAHRENHEIT as TEMP_FAHRENHEIT, TIME_MINUTES as TIME_MINUTES
from homeassistant.core import HomeAssistant as HomeAssistant, callback as callback
from homeassistant.helpers.dispatcher import async_dispatcher_connect as async_dispatcher_connect
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.update_coordinator import DataUpdateCoordinator as DataUpdateCoordinator
from typing import Any

SENSOR_KIND_BATTERY: str
SENSOR_KIND_TEMPERATURE: str
SENSOR_KIND_UPTIME: str
SENSOR_ATTRS_MAP: Any
PAIRED_SENSOR_SENSORS: Any
VALVE_CONTROLLER_SENSORS: Any

async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry, async_add_entities: AddEntitiesCallback) -> None: ...

class PairedSensorSensor(PairedSensorEntity, SensorEntity):
    _attr_unit_of_measurement: Any
    def __init__(self, entry: ConfigEntry, coordinator: DataUpdateCoordinator, kind: str, name: str, device_class: Union[str, None], icon: Union[str, None], unit: Union[str, None]) -> None: ...
    _attr_state: Any
    def _async_update_from_latest_data(self) -> None: ...

class ValveControllerSensor(ValveControllerEntity, SensorEntity):
    _attr_unit_of_measurement: Any
    def __init__(self, entry: ConfigEntry, coordinators: dict[str, DataUpdateCoordinator], kind: str, name: str, device_class: Union[str, None], icon: Union[str, None], unit: Union[str, None]) -> None: ...
    async def _async_continue_entity_setup(self) -> None: ...
    _attr_available: Any
    _attr_state: Any
    def _async_update_from_latest_data(self) -> None: ...



from homeassistant.components.sensor import PLATFORM_SCHEMA as PLATFORM_SCHEMA, SensorEntity as SensorEntity
from homeassistant.const import CONF_NAME as CONF_NAME, CONF_UNIT_OF_MEASUREMENT as CONF_UNIT_OF_MEASUREMENT, DEVICE_CLASS_TIMESTAMP as DEVICE_CLASS_TIMESTAMP
from homeassistant.core import HomeAssistant as HomeAssistant
from homeassistant.helpers.entity_platform import AddEntitiesCallback as AddEntitiesCallback
from homeassistant.helpers.typing import ConfigType as ConfigType, DiscoveryInfoType as DiscoveryInfoType
from typing import Any

DEFAULT_NAME: str

async def async_setup_platform(hass: HomeAssistant, config: ConfigType, async_add_entities: AddEntitiesCallback, discovery_info: Union[DiscoveryInfoType, None] = ...) -> None: ...

class UptimeSensor(SensorEntity):
    _attr_name: Any
    _attr_device_class: Any
    _attr_should_poll: bool
    _attr_state: Any
    def __init__(self, name: str) -> None: ...



from . import DOMAIN as DOMAIN
from homeassistant.components.device_automation.exceptions import InvalidDeviceAutomationConfig as InvalidDeviceAutomationConfig
from homeassistant.const import CONF_ABOVE as CONF_ABOVE, CONF_BELOW as CONF_BELOW, CONF_ENTITY_ID as CONF_ENTITY_ID, CONF_TYPE as CONF_TYPE, DEVICE_CLASS_BATTERY as DEVICE_CLASS_BATTERY, DEVICE_CLASS_CO as DEVICE_CLASS_CO, DEVICE_CLASS_CO2 as DEVICE_CLASS_CO2, DEVICE_CLASS_CURRENT as DEVICE_CLASS_CURRENT, DEVICE_CLASS_ENERGY as DEVICE_CLASS_ENERGY, DEVICE_CLASS_HUMIDITY as DEVICE_CLASS_HUMIDITY, DEVICE_CLASS_ILLUMINANCE as DEVICE_CLASS_ILLUMINANCE, DEVICE_CLASS_POWER as DEVICE_CLASS_POWER, DEVICE_CLASS_POWER_FACTOR as DEVICE_CLASS_POWER_FACTOR, DEVICE_CLASS_PRESSURE as DEVICE_CLASS_PRESSURE, DEVICE_CLASS_SIGNAL_STRENGTH as DEVICE_CLASS_SIGNAL_STRENGTH, DEVICE_CLASS_TEMPERATURE as DEVICE_CLASS_TEMPERATURE, DEVICE_CLASS_VOLTAGE as DEVICE_CLASS_VOLTAGE
from homeassistant.core import HomeAssistant as HomeAssistant, HomeAssistantError as HomeAssistantError, callback as callback
from homeassistant.helpers import condition as condition
from homeassistant.helpers.entity import get_device_class as get_device_class, get_unit_of_measurement as get_unit_of_measurement
from homeassistant.helpers.entity_registry import async_entries_for_device as async_entries_for_device, async_get_registry as async_get_registry
from homeassistant.helpers.typing import ConfigType as ConfigType
from typing import Any

DEVICE_CLASS_NONE: str
CONF_IS_BATTERY_LEVEL: str
CONF_IS_CO: str
CONF_IS_CO2: str
CONF_IS_CURRENT: str
CONF_IS_ENERGY: str
CONF_IS_HUMIDITY: str
CONF_IS_ILLUMINANCE: str
CONF_IS_POWER: str
CONF_IS_POWER_FACTOR: str
CONF_IS_PRESSURE: str
CONF_IS_SIGNAL_STRENGTH: str
CONF_IS_TEMPERATURE: str
CONF_IS_VOLTAGE: str
CONF_IS_VALUE: str
ENTITY_CONDITIONS: Any
CONDITION_SCHEMA: Any

async def async_get_conditions(hass: HomeAssistant, device_id: str) -> list[dict[str, str]]: ...
def async_condition_from_config(config: ConfigType, config_validation: bool) -> condition.ConditionCheckerType: ...
async def async_get_condition_capabilities(hass, config): ...
